C$$$  SUBPROGRAM DOCUMENTATION BLOCK
C                .      .    .                                       .
C SUBPROGRAM:    GTNEWMAP    INPUT NEW MAP DATA TO BE QC'ED
C   PRGMMR: M. BARTH         ORG: FSL        DATE: 02-04-22
C
C ABSTRACT: INPUT NEW MAP DATA TO BE QC'ED FROM THE COMPLETE SET OF
C   NETCDF FILES THAT MAY GO INTO ONE HALF-HOUR'S WORTH OF OUTPUT.
C
C PROGRAM HISTORY LOG:
C   02-04-22  M. BARTH    ORIGINAL VERSION 
C   02-06-26  M. BARTH    ADDED DTIME
C   08-06-20  L. BENJAMIN CHANGED MADISCON TO INTQC1CON2.
C   12-07-01  L. BENJAMIN ADDED 6 MINUTE RESOLUTION DATA
C   12-07-06  L. BENJAMIN FIXED 6 MINUTE ADD BUG IN FTIME
C   12-08-03  L. BENJAMIN EVERY OTHER DP FOR 6 MINUTE DATA ENDING 00
C   12-09-19  L. BENJAMIN EVERY OTHER DP FOR 6 MINUTE DATA ENDING 00
C
C USAGE:    CALL GTNEWMAP (ATIME,NDLEV,NDSTA,NDVAR,NRECS,U,V,T,HT,LT,
C                          STAID,DTIME,QCAPP,QCRES,ICAPP,ICRES,QCDESC,
C                          RSAQC,STATUS)
C
C   INPUT ARGUMENT LIST:
C     ATIME    - C       YYJJJHHMM OF TIME TO BE PROCESSED
C     NDLEV    - I       SIZE TO USE FOR LEVEL DIMENSION IN ARRAYS
C     NDSTA    - I       SIZE TO USE FOR STATION DIMENSION IN ARRAYS
C     NDVAR    - I       SIZE TO USE FOR VARIABLE DIMENSION IN ARRAYS
C
C   OUTPUT ARGUMENT LIST:
C     NRECS    - I       NUMBER OF RECORDS IN THE OUTPUT FILE
C     U        - RA      U WIND COMPONENTS
C     V        - RA      V WIND COMPONENTS
C     T        - RA      VIRTUAL TEMPERATURES
C     HT       - RA      HEIGHTS
C     LT       - RA      LEVEL TYPES
C     STAID    - CA      STATION NAMES
C     DTIME    - DA      STORED OB TIMES (NOT I4TIMES)
C     QCAPP    - IA      QC APPLIED WORDS
C     QCRES    - IA      QC RESULTS WORDS
C     ICAPP    - IA      IC (INTERNAL CONSISTENCY CHECK) APPLIED
C                        WORDS
C     ICRES    - IA      IC RESULTS WORDS
C     QCDESC   - CA      QC DATA DESCRIPTORS
C     RSAQC    - IA      RSA-STYLE QC FLAGS
C     STATUS   - I       SUCCESS_P OR FAILURE_P IF INPUT FILE PROBLEM
C
C   INPUT FILES:
C     QC FILE  - QC_DIR/FILENAME, FILENAME IS CREATED FROM ATIME
C
C SUBPROGRAMS CALLED:
C     UNIQUE:  - MKMAPFILE, RDMAPOBS
C     NETCDF:  - NCPOPT
C     MADISLIB:- M_MKI4TM, MCHGTIM
C
C REMARKS:
C   NONE
C ATTRIBUTES:
C   LANGUAGE: FORTRAN-77 + EXTENSIONS
C   MACHINE:  UNIX
C$$$






      SUBROUTINE GTNEWMAP (ATIME,NDLEV,NDSTA,NDVAR,NRECS,U,V,T,HT,LT,
     1                     STAID,DTIME,QCAPP,QCRES,ICAPP,ICRES,QCDESC,
     2                     RSAQC,STATUS)





C     ------------------------------------------------------------------
C     INCLUDES AND COMMONS
C     ------------------------------------------------------------------
      INCLUDE 'IMPLICIT'
      INCLUDE 'INTQC1CON2'
      INCLUDE 'STDOUT'


C     ------------------------------------------------------------------
C     ARGUMENTS
C     ------------------------------------------------------------------
      CHARACTER*9   ATIME
      INTEGER       NDLEV
      INTEGER       NDSTA
      INTEGER       NDVAR
      INTEGER       NRECS
      REAL          U(NDLEV,NDSTA)
      REAL          V(NDLEV,NDSTA)
      REAL          T(NDSTA)
      REAL          HT(NDSTA)
      REAL          LT(NDSTA)
      CHARACTER*6   STAID(NDSTA)
      DOUBLE PRECISION DTIME(NDSTA)
      INTEGER       QCAPP(NDLEV,NDSTA,NDVAR)
      INTEGER       QCRES(NDLEV,NDSTA,NDVAR)
      INTEGER       ICAPP(NDLEV,NDSTA,NDVAR)
      INTEGER       ICRES(NDLEV,NDSTA,NDVAR)
      CHARACTER*1   QCDESC(NDLEV,NDSTA,NDVAR)
      INTEGER       RSAQC(NDLEV,NDSTA,NDVAR)
      INTEGER       STATUS


C     ------------------------------------------------------------------
C     LOCAL
C     ------------------------------------------------------------------
      CHARACTER*4 CRES
      CHARACTER*9 FTIME,FTIME6
      INTEGER I,LSTRCQ,NRECW,NRECT,NRECI,SMIN,EMIN,I4TIME,SMIN6

C     *****************************************************************
C     ******************* BEGIN OF GTNEWMAP ***************************
C     *****************************************************************

      LSTRCQ = 0
      NRECS = 0

C     ------------------------------------------------------------------
C     KEEP NETCDF ERRORS FROM KILLING THE PROGRAM.
C     ------------------------------------------------------------------
      CALL NCPOPT(0)

C     ------------------------------------------------------------------
C     LOOP FOR EACH 5 MINUTE PERIOD IN THE HALF HOUR.  HERE'S THE INPUT
C     FILE SCHEDULE:
C
C     MINUTE            CRES           FILES TO EXPECT     
C     ------            ----           ---------------     
C     0,5,10,...,55      0005           WIND                 
C     0,6,12,...,54      0006           WIND                 
C     0,10,20,...,50     0010           WIND
C     0,15,30,45         0015           WIND, TEMP
C     0,30               0030           WIND, TEMP
C     0,20,40            0020           WIND, TEMP
C     0                  0100           WIND, TEMP
C     
C     HERE'S WHAT GOES INTO EACH HALF-HOUR, USING THE EXAMPLE OF ATIME
C     = 1200 AND 1230:
C
C     FTIME = INPUT         ATIME = OUTPUT
C     -------------         --------------
C     1135 - 1200           1200
C     1205 - 1230           1230                   
C     ------------------------------------------------------------------
      CALL M_MKI4TM(ATIME,I4TIME,STATUS)
      IF(ATIME(8:9).EQ.'00')THEN
         
         SMIN = 35
         EMIN = 60
         SMIN6 = 36
         CALL MCHGTIM(ATIME,-1800,FTIME,STATUS)

      ELSE IF(ATIME(8:9).EQ.'30')THEN

         SMIN = 5
c$$$         SMIN6 = 6
         SMIN6 = 12
         EMIN = 30
         FTIME = ATIME(1:7)//'00'

      ENDIF
      FTIME6 = FTIME

      DO 1 I = SMIN, EMIN, 5

C     ------------------------------------------------------------------
C     DO THE 5-MINUTE RESOLUTION
C     ------------------------------------------------------------------
         CRES = '0005'
         IF(I.LT.10)THEN
            WRITE(FTIME(9:9),2)I
 2          FORMAT(I1)
         ELSE IF(I.NE.60)THEN
            WRITE(FTIME(8:9),3)I
 3          FORMAT(I2)
         ELSE 
            FTIME = ATIME
         ENDIF
         CALL MKMAPFILE(FTIME,CRES,ATIME,LSTRCQ,NRECW,NRECT,NRECI,
     1                  STATUS)
         WRITE(STDOUT,4)FTIME,CRES,NRECW,NRECT,NRECI,STATUS
 4       FORMAT('  - ',A,1X,A,' NRECW ',I3,' NRECT ',I3,' NRECI ',I3,
     1          '  STATUS ',I5)
         IF(STATUS.NE.SUCCESS_P)GO TO 900
         NRECS = NRECS + NRECI

C     ------------------------------------------------------------------
C     DO THE 10-MINUTE RESOLUTION
C     ------------------------------------------------------------------
         IF(I/10*10.EQ.I)THEN

            CRES = '0010'
            IF(I.LT.10)THEN
               WRITE(FTIME(9:9),2)I
            ELSE IF(I.NE.60)THEN
               WRITE(FTIME(8:9),3)I
            ELSE
               FTIME = ATIME
            ENDIF
            CALL MKMAPFILE(FTIME,CRES,ATIME,LSTRCQ,NRECW,NRECT,NRECI,
     1                     STATUS)
            WRITE(STDOUT,4)FTIME,CRES,NRECW,NRECT,NRECI,STATUS
            IF(STATUS.NE.SUCCESS_P)GO TO 900
            NRECS = NRECS + NRECI

         ENDIF

C     ------------------------------------------------------------------
C     DO THE 15-MINUTE RESOLUTION
C     ------------------------------------------------------------------
         IF(I/15*15.EQ.I)THEN

            CRES = '0015'
            IF(I.LT.10)THEN
               WRITE(FTIME(9:9),2)I
            ELSE IF(I.NE.60)THEN
               WRITE(FTIME(8:9),3)I
            ELSE
               FTIME = ATIME
            ENDIF
            CALL MKMAPFILE(FTIME,CRES,ATIME,LSTRCQ,NRECW,NRECT,NRECI,
     1                     STATUS)
            WRITE(STDOUT,4)FTIME,CRES,NRECW,NRECT,NRECI,STATUS
            IF(STATUS.NE.SUCCESS_P)GO TO 900
            NRECS = NRECS + NRECI

         ENDIF

C     ------------------------------------------------------------------
C     DO THE 20-MINUTE RESOLUTION
C     ------------------------------------------------------------------
         IF(I/20*20.EQ.I)THEN

            CRES = '0020'
            IF(I.LT.10)THEN
               WRITE(FTIME(9:9),2)I
            ELSE IF(I.NE.60)THEN
               WRITE(FTIME(8:9),3)I
            ELSE
               FTIME = ATIME
            ENDIF
            CALL MKMAPFILE(FTIME,CRES,ATIME,LSTRCQ,NRECW,NRECT,NRECI,
     1                     STATUS)
            WRITE(STDOUT,4)FTIME,CRES,NRECW,NRECT,NRECI,STATUS
            IF(STATUS.NE.SUCCESS_P)GO TO 900
            NRECS = NRECS + NRECI

         ENDIF

C     ------------------------------------------------------------------
C     DO THE 30-MINUTE RESOLUTION
C     ------------------------------------------------------------------
         IF(I/30*30.EQ.I)THEN

            CRES = '0030'
            IF(I.LT.10)THEN
               WRITE(FTIME(9:9),2)I
            ELSE IF(I.NE.60)THEN
               WRITE(FTIME(8:9),3)I
            ELSE
               FTIME = ATIME
            ENDIF
            CALL MKMAPFILE(FTIME,CRES,ATIME,LSTRCQ,NRECW,NRECT,NRECI,
     1                     STATUS)
            WRITE(STDOUT,4)FTIME,CRES,NRECW,NRECT,NRECI,STATUS
            IF(STATUS.NE.SUCCESS_P)GO TO 900
            NRECS = NRECS + NRECI

         ENDIF

C     ------------------------------------------------------------------
C     DO THE HOURLY RESOLUTION
C     ------------------------------------------------------------------
         IF(I.EQ.60)THEN

            CRES = '0100'
            FTIME = ATIME
            CALL MKMAPFILE(FTIME,CRES,ATIME,LSTRCQ,NRECW,NRECT,NRECI,
     1                     STATUS)
            WRITE(STDOUT,4)FTIME,CRES,NRECW,NRECT,NRECI,STATUS
            IF(STATUS.NE.SUCCESS_P)GO TO 900
            NRECS = NRECS + NRECI

         ENDIF

 1    CONTINUE

      FTIME = FTIME6
c$$$      DO 111 I = SMIN6, EMIN, 6
      DO 111 I = SMIN6, EMIN, 12

C     ------------------------------------------------------------------
C     DO THE 6-MINUTE RESOLUTION
C     ------------------------------------------------------------------
         CRES = '0006'
         IF(I.LT.10)THEN
            WRITE(FTIME(9:9),2)I
         ELSE IF(I.NE.60)THEN
            WRITE(FTIME(8:9),3)I
         ELSE 
            FTIME = ATIME
         ENDIF
         CALL MKMAPFILE(FTIME,CRES,ATIME,LSTRCQ,NRECW,NRECT,NRECI,
     1                  STATUS)
         WRITE(STDOUT,4)FTIME,CRES,NRECW,NRECT,NRECI,STATUS
         IF(STATUS.NE.SUCCESS_P)GO TO 900
         NRECS = NRECS + NRECI
 111  CONTINUE



C     ------------------------------------------------------------------
C     IF THERE ARE OBS TO BE PROCESSED, READ IN THOSE OBS (& QC INFO
C     FROM ANY PAST RUNS).
C     ------------------------------------------------------------------
      IF(NRECS.NE.0)THEN

         WRITE(STDOUT,*)'  - CALLING RDMAPOBS'
         CALL RDMAPOBS(ATIME,'QC_DIR',NDLEV,NDSTA,NDVAR,NRECS,U,V,T,HT,
     1                 LT,STAID,DTIME,QCAPP,QCRES,ICAPP,ICRES,QCDESC,
     2                 RSAQC,STATUS)

      ENDIF
C
 900  IF(STATUS.NE.SUCCESS_P)NRECS = 0

      RETURN
      END
