C$$$  MAIN PROGRAM DOCUMENTATION BLOCK
C                .      .    .                                       .
C MAIN PROGRAM:  MAP         "MAP" DATASET STAGE 1 & 2 QC PROCESSING
C   PRGMMR: M. BARTH          ORG: FSL        DATE: 02-04-22
C
C ABSTRACT:
C   PERFORMS OR REFORMATS STAGE 1 AND STAGE 2 QC CHECKS ON "MAP"
C   (MULTI-AGENCY PROFILER) FILES.  THESE CHECKS ARE STAGE 1:  VALIDITY 
C   CHECK, STAGE 2: INTERNAL CONSISTENCY CHECK, TIME-HEIGHT CONSISTENCY 
C   CHECK.
C
C PROGRAM HISTORY LOG:
C   02-04-22  M. BARTH  ORIGINAL VERSION.
C   02-06-26  M. BARTH  ADDED OUTPUT OF LDAD SFC FILES TO CSV_DIR.
C   06-09-01  L. BENJAMIN ADDED DYNAMIC SETTING NUMBER OF TEMP LEVELS.
C                         MXTLVL IS MAX POSSIBLE AND NMTLVL IS NUMBER
C                         IN NETCDF FILE.  ADDED FILE (NUMTEMPGATES)
C                         WHICH SETS NMTLVL VARIABLE.  THE DEFAULT SIZE
C                         IS 30.
C   08-06-20  L. BENJAMIN CHANGED MADISCON TO INTQC1CON2 AND 
C                         MAPCON.  ADDED INTQC1COM1.  REMOVED MINIT 
C                         CALL, SET COMMON VARIABLES LOCALLY.
C   08-08-19  L. BENJAMIN CHANGED FMC_P TO MC_P
C
C USAGE:
C   INPUT FILES: (FOR THE ENTIRE PROGRAM)
C     ACCEPTIN - LIST OF STATIONS/LEVELS TO BE ACCEPTED
C     MAP.INI  - MISCELLANEOUS INPUT PARAMETERS
C     MAPVARS  - FILE LISTING NON-SPECIAL-HANDLING VARIABLES
C     NUMTEMPGATES - NUMBER OF RASS GATES IN INPUT FILE
C     QCTIME   - ASCII STRING CONTAINING THE DATE AND TIME IN FORMAT
C                YYDDDHHMM (YEAR-JULIAN DAY-HOUR-MINUTE.)
C     RAW FILES- RAW (WITHOUT QC) DATA FILES, FOR WIND & TEMP.  
C                (FILENAMES ARE CREATED IN SUBROUTINE THAT READS FILES.)
C     REJECTIN - LIST OF STATIONS/LEVELS TO BE REJECTED
C     QC FILE  - QC'ED DATA FILE.  (FILENAME IS CREATED IN SUB-
C                ROUTINE THAT READS/WRITES THE FILE.)
C     QC_CDL   - CDL FOR A NEW QC FILE
C
C   OUTPUT FILES:
C     QC FILE  - QC'ED DATA FILE.
C     CSV FILES- CSV FILES OF SFC DATA FOR LDAD (FILENAMES ARE CREATED
C                IN SUBROUTINE THAT WRITES THE FILES)
C     STDOUT   - STANDARD OUTPUT FILE
C     STDERR   - STANDARD ERROR FILE
C
C SUBPROGRAMS CALLED:
C     UNIQUE:  - GTNEWMAP, MKSMRYMAP, SMRYRPT, UAWINDVC, UATEMPVC,
C                WTMAP, MAPSFCCSV
C     MADISLIB:- M_CHCKQC, M_CLRQC, MINIT, M_GETLUN, M_SETQC
C
C REMARKS:
C     NONE.
C
C ATTRIBUTES:
C   LANGUAGE: FORTRAN 77 + EXTENSIONS
C   MACHINE: UNIX
C
C$$$






      PROGRAM MAP


C     ------------------------------------------------------------------
C     INCLUDES AND COMMONS
C     ------------------------------------------------------------------
      INCLUDE 'IMPLICIT'
      INCLUDE 'INTQC1CON2'
      INCLUDE 'MAPCON'
      INCLUDE 'INTQC1COM1'
      INCLUDE 'STDOUT'
      INCLUDE 'MAPCOM'
c$$$      INCLUDE 'MAPCONLOC'


C     ------------------------------------------------------------------
C     LOCAL PARAMETERS USED FOR DIMENSIONING ARRAYS.
C     ------------------------------------------------------------------
      INTEGER       NDSTA
      INTEGER       NDCHK
      INTEGER       NDLEV
      INTEGER       NDVAR
      PARAMETER     (NDSTA = MXMAP_P)
      PARAMETER     (NDLEV = MXMAPLEV_P)
      PARAMETER     (NDVAR = 2)

      CHARACTER*9   ATIME
      INTEGER       STATUS
      INTEGER       NRECS
      INTEGER       I,J,K,LUN,VAL1,VAL2,QCCHKS
      REAL          SPD,DIR
      INTEGER       UAWINDVC,UATEMPVC
      REAL          U(NDLEV,NDSTA)
      REAL          V(NDLEV,NDSTA)
      REAL          T(NDLEV,NDSTA)
      REAL          HT(NDLEV,NDSTA)
      REAL          LT(NDLEV,NDSTA)
      CHARACTER*6   STAID(NDSTA)
      DOUBLE PRECISION DTIME(NDSTA)
      LOGICAL       PROC_STA(NDSTA)
      LOGICAL       EXISTS
      INTEGER       LUN2

      INTEGER       QCAPP(NDLEV,NDSTA,NDVAR)
      INTEGER       QCRES(NDLEV,NDSTA,NDVAR)
      INTEGER       ICAPP(NDLEV,NDSTA,NDVAR)
      INTEGER       ICRES(NDLEV,NDSTA,NDVAR)
      INTEGER       RSAQC(NDLEV,NDSTA,NDVAR)
      CHARACTER*1   QCDESC(NDLEV,NDSTA,NDVAR)

      INTEGER       DBG(2)
      CHARACTER*3   LSTR
      INTEGER       DUMMY

C     PRIVATE MAP CODE NUMBERS
      INTEGER MAX_MAP_QCCHK_L
      PARAMETER (MAX_MAP_QCCHK_L = 11)

C     ------------------------------------------------------------------
C     DATA STATEMENTS (KEEP AT BOTTOM OF DECLARATIONS)
C     ------------------------------------------------------------------
      DATA STDOUT /6/
      DATA STDIN  /5/
      DATA STDERR /0/
      DATA DIR /0.0/

C     *****************************************************************
C     ****************** BEGIN OF PROGRAM MAP *************************
C     *****************************************************************

C     -----------------------------------------------------------------
C     INITIALIZE THE MADIS LIBRARY
C     -----------------------------------------------------------------
      VERBOSE = .TRUE.
      CDTYPE = EXTERNAL_DS_P
      CDBTYPE = FSL_DB_P
      MISS_AWIPS_VAL2 = -9999
      MISS_ACARS_VAL2 = 3.40282346E+38

C     -----------------------------------------------------------------
C     OPEN OUTPUT FILES FOR STDOUT AND STDERR
C     -----------------------------------------------------------------
      OPEN(UNIT=STDOUT,FILE='MAP.OUT',STATUS='UNKNOWN')
      OPEN(UNIT=STDERR,FILE='MAP.ERR',STATUS='UNKNOWN')

C     -----------------------------------------------------------------
C     GET DEBUGGING FLAGS
C     -----------------------------------------------------------------
      OPEN(UNIT=1,FILE='MAP.INI',STATUS='UNKNOWN')
      DO 1 I = 1, 2
         READ(1,*)DBG(I)
 1    CONTINUE
      CLOSE(UNIT=1)

C     ------------------------------------------------------------------
C     OPEN THE FILE POINTED TO BY 'QCTIME' AND GET THE TIME.
C     OF THE DATA TO BE QC'ED.  
C     ------------------------------------------------------------------
      WRITE(STDOUT,*)'- OPENING QCTIME'
      CALL M_GETLUN(LUN, 1)
      OPEN(UNIT=LUN,FILE='QCTIME',STATUS='UNKNOWN')
      READ(LUN,101)ATIME
 101  FORMAT(A)
      WRITE(STDOUT,*)'- QCTIME IS: ',ATIME
      CLOSE(UNIT=LUN)
      CALL M_GETLUN(LUN, 2)

C     -----------------------------------------------------------------
C     SET NMTLVL VALUE BASE INPUT FILE
C     -----------------------------------------------------------------
      WRITE(STDOUT,*)'- SETTING # TEMP LEVELS BASED ON PAR FILE '
      NMTLVL = 30
      INQUIRE(FILE='NUMTEMPGATES', EXIST=EXISTS)
      IF(EXISTS)THEN
         CALL M_GETLUN(LUN2, 1)
         OPEN(UNIT=LUN2,FILE='NUMTEMPGATES',STATUS='UNKNOWN')
         READ(LUN2,*)NMTLVL
         IF(NMTLVL.LT.15 .OR. NMTLVL.GE.100)THEN
            NMTLVL = 15
         ENDIF
         CALL M_GETLUN(LUN2, 2)
      ENDIF
    
C     -----------------------------------------------------------------
C     GET THE OBS FOR THE CURRENT HOUR.
C     -----------------------------------------------------------------
      WRITE(STDOUT,*)'- CALLING GTNEWMAP FOR TIME ',ATIME
      CALL GTNEWMAP(ATIME,NDLEV,NDSTA,NDVAR,NRECS,U,V,T,HT,LT,
     1              STAID,DTIME,QCAPP,QCRES,ICAPP,ICRES,QCDESC,RSAQC,
     2              STATUS)
      IF (STATUS .NE. SUCCESS_P) GO TO 900

      WRITE(STDOUT,*)'- NUMBER OF CURRENT OB RECORDS ',NRECS

C     -----------------------------------------------------------------
C     IF NOTHING TO PROCESS:  WE'VE SUCCESSFULLY FINISHED.
C     -----------------------------------------------------------------
      IF(NRECS.EQ.0)GO TO 1000

C     -----------------------------------------------------------------
C     - APPLY THE VALIDITY CHECKS ON WIND SPEED AND VIRTUAL TEMP.
C     - CREATE THE MASTER BITS.
C     -----------------------------------------------------------------
      DO 100 J = 1, NRECS

         PROC_STA(J) = .TRUE.

         DO 200 I = 1, NDLEV

            DO 300 K = 1, 2

               IF((K.EQ.1.AND.U(I,J).NE.MISS_P.AND.V(I,J).NE.MISS_P).OR.
     1            (K.EQ.2.AND.T(I,J).NE.MISS_P))THEN

                  IF(K.EQ.1)THEN

C     -----------------------------------------------------------------
C     APPLY THE VALIDITY CHECK TO WIND SPEED ONLY (WE KNOW DIRECTION IS
C     VALID, AS WE CALCULATE IT -- SO WE JUST SET IT TO 0.)  ALSO NOTE
C     THE UAWINDVC ROUTINE EXPECTS M/S FOR SPD -- WE HAVE KTS, SO 
C     WE'LL CONVERT BEFORE SENDING INTO THE ROUTINE.
C     -----------------------------------------------------------------

                     SPD = SQRT(U(I,J)*U(I,J)+V(I,J)*V(I,J))/1.943861286
                     VAL1 = UAWINDVC(1,HT(I,J),SPD,DIR)
                     CALL M_SETQC(QCAPP(I,J,K),VC_P,QCAPP(I,J,K),STATUS)
                     IF(VAL1.GT.0)CALL M_SETQC(QCRES(I,J,K),VC_P,
     1                                         QCRES(I,J,K),STATUS)

                  ELSE

                     VAL1 = UATEMPVC(1,HT(I,J),T(I,J))
                     CALL M_SETQC(QCAPP(I,J,K),VC_P,QCAPP(I,J,K),STATUS)
                     IF(VAL1.GT.0)CALL M_SETQC(QCRES(I,J,K),VC_P,
     1                                         QCRES(I,J,K),STATUS)

                  ENDIF
               ENDIF

C     -----------------------------------------------------------------
C     CREATE THE MASTER BITS.
C     -----------------------------------------------------------------
               IF(QCAPP(I,J,K).NE.0)THEN
                  CALL M_SETQC(QCAPP(I,J,K),MASTER_P,QCAPP(I,J,K),
     1                         STATUS)
                  ELSE
                  CALL M_CLRQC(QCAPP(I,J,K),MASTER_P,QCAPP(I,J,K),
     1                         STATUS)
               ENDIF
               IF(QCRES(I,J,K).NE.0)THEN
                  CALL M_SETQC(QCRES(I,J,K),MASTER_P,QCRES(I,J,K),
     1                         STATUS)
               ELSE
                  CALL M_CLRQC(QCRES(I,J,K),MASTER_P,QCRES(I,J,K),
     1                         STATUS)
               ENDIF

 300        CONTINUE

 200     CONTINUE

 100  CONTINUE

C     -----------------------------------------------------------------
C     MAKE THE DESCRIPTORS.
C     -----------------------------------------------------------------
      WRITE(STDOUT,*)'- CALLING MKSMRYMAP FOR WIND'
      CALL MKSMRYMAP(1,NDLEV,NDSTA,NRECS,STAID,QCAPP,QCRES,QCDESC,RSAQC)

      WRITE(STDOUT,*)'- CALLING MKSMRYMAP FOR TEMP'
      CALL MKSMRYMAP(2,NDLEV,NDSTA,NRECS,STAID,QCAPP(1,1,2),
     1               QCRES(1,1,2),QCDESC(1,1,2),RSAQC(1,1,2))

C     -----------------------------------------------------------------
C     CALCULATE QC STATS AND REPORT THEM TO STDOUT
C     -----------------------------------------------------------------
      QCCHKS = 0
      CALL M_SETQC(QCCHKS,VC_P,QCCHKS,STATUS)
      CALL M_SETQC(QCCHKS,IC_P,QCCHKS,STATUS)
      CALL M_SETQC(QCCHKS,THC_P,QCCHKS,STATUS)
      CALL M_SETQC(QCCHKS,MC_P,QCCHKS,STATUS)
      WRITE(STDOUT,*)'- CALLING SMRYRPT FOR WIND'
      CALL SMRYRPT(6,NDLEV,NDSTA,MAX_MAP_QCCHK_L,QCCHKS,'WIND',MISS_P,
     1             STAID,PROC_STA,NRECS,QCAPP,QCRES,DUMMY)

      WRITE(STDOUT,*)'- CALLING SMRYRPT FOR TEMP'
      CALL SMRYRPT(6,NDLEV,NDSTA,MAX_MAP_QCCHK_L,QCCHKS,'TEMP',MISS_P,
     1             STAID,PROC_STA,NRECS,QCAPP(1,1,2),QCRES(1,1,2),DUMMY)

C     -----------------------------------------------------------------
C     ECHO THE OBS AND QC RESULTS IF DESIRED FOR DEBUGGING
C     -----------------------------------------------------------------
      IF(DBG(1).NE.0.OR.DBG(2).NE.0)THEN

C     -----------------------------------------------------------------
C     DO THE WINDS AND THE TEMPS SEPARATELY.
C     -----------------------------------------------------------------
         DO 1500 J = 1, NRECS
            IF(DBG(1).NE.0)THEN
               WRITE(STDOUT,*)'WIND OBS FOR ',STAID(J)(1:5)
            ELSE
               WRITE(STDOUT,*)'BAD WIND OBS FOR ',STAID(J)(1:5)
            ENDIF

            DO 1502 I = 1, NDLEV

               IF((DBG(1).EQ.0.AND.QCRES(I,J,1).EQ.0).OR.
     1            (HT(I,J).EQ.MISS_P))GO TO 1502

               WRITE(LSTR,502)I
 502           FORMAT(I3)
               IF(LSTR(1:1).EQ.' ')LSTR(1:1) = '0'
               IF(LSTR(2:2).EQ.' ')LSTR(2:2) = '0'
               WRITE(STDOUT,503)J,STAID(J)(1:5),LSTR,INT(LT(I,J)),
     1                          HT(I,J),U(I,J),V(I,J),QCAPP(I,J,1),
     2                          QCRES(I,J,1),ICAPP(I,J,1),ICRES(I,J,1),
     3                          QCDESC(I,J,1)
 503           FORMAT(1X,I4,1X,A5,1X,' L ',A,1X,'M',I1,1X,F8.2,' U ',
     1                F9.2,' V ',F9.2,' QCA ',I4,' QCR ',I4,' ICA ',I4,
     2                ' ICR ',I4,' DD ',A1)
 1502       CONTINUE

 1500    CONTINUE

         DO 2500 J = 1, NRECS
            IF(DBG(1).NE.0)THEN
               WRITE(STDOUT,*)'TEMP OBS FOR ',STAID(J)(1:5)
            ELSE
               WRITE(STDOUT,*)'BAD TEMP OBS FOR ',STAID(J)(1:5)
            ENDIF

            DO 2502 I = 1, NDLEV

               IF((DBG(1).EQ.0.AND.QCRES(I,J,2).EQ.0).OR.
     1            (HT(I,J).EQ.MISS_P))GO TO 2502

               WRITE(LSTR,502)I
               IF(LSTR(1:1).EQ.' ')LSTR(1:1) = '0'
               WRITE(STDOUT,2503)J,STAID(J)(1:5),LSTR,HT(I,J),T(I,J),
     1                           QCAPP(I,J,2),QCRES(I,J,2),
     2                          ICAPP(I,J,2),ICRES(I,J,2),QCDESC(I,J,2)
 2503          FORMAT(1X,I4,1X,A5,1X,' L ',A,1X,F9.2,' T ',
     1                F9.2,' QCA ',I4,' QCR ',I4,' ICA ',I4,
     2                ' ICR ',I4,' DD ',A1)
 2502       CONTINUE

 2500    CONTINUE

      ENDIF


C     -----------------------------------------------------------------
C     OUTPUT THE RESULTS TO THE QC FILE
C     -----------------------------------------------------------------
      WRITE(STDOUT,*)'- CALLING WTMAP FOR TIME ',ATIME
      CALL WTMAP(ATIME,NDLEV,NDSTA,NDVAR,NRECS,QCAPP,QCRES,ICAPP,ICRES,
     1           QCDESC,RSAQC,STATUS)
      IF (STATUS .NE. SUCCESS_P) GO TO 900

C     -----------------------------------------------------------------
C     INPUT/PROCESS/OUTPUT LDAD SFC FILES.
C     -----------------------------------------------------------------
      WRITE(STDOUT,*)'- CALLING MAPSFCCSV FOR TIME ',ATIME
      CALL MAPSFCCSV(ATIME,NRECS,STAID,DTIME,STATUS)
      IF (STATUS .NE. SUCCESS_P) GO TO 900

 1000 CONTINUE


C     -----------------------------------------------------------------
C     OUTPUT THE FAREWELL MESSAGE
C     -----------------------------------------------------------------
 900  IF (STATUS .EQ. SUCCESS_P) THEN
        IF(NRECS .NE. 0)THEN
           WRITE (STDOUT,5070)
 5070      FORMAT(/,' - MAP FILE WAS CREATED OR MODIFIED')
        ENDIF
        WRITE (STDOUT, 5080)
5080    FORMAT (///35X,
     1   '<<>><<>><<>><<>><<>><<>><<>><<>><<>><<>>'/35X,
     2   '<<>>       PROGRAM     MAP          <<>>'/35X,
     3   '<<>>         NORMAL TERMINATION     <<>>'/35X,
     4   '<<>><<>><<>><<>><<>><<>><<>><<>><<>><<>>')

      ELSE
        WRITE (STDOUT, 5090)
5090    FORMAT (///35X,
     1   '<<>><<>><<>><<>><<>><<>><<>><<>><<>><<>>'/35X,
     2   '<<>>       PROGRAM     MAP          <<>>'/35X,
     3   '<<>>       ABNORMAL TERMINATION     <<>>'/35X,
     4   '<<>><<>><<>><<>><<>><<>><<>><<>><<>><<>>')
      ENDIF



C     -----------------------------------------------------------------
C     CLOSE STDOUT AND STDERR
C     -----------------------------------------------------------------
      CLOSE(STDOUT)
      CLOSE(STDERR)

      STOP
      END

