C$$$  SUBPROGRAM DOCUMENTATION BLOCK
C                .      .    .                                       .
C SUBPROGRAM:    MAPLVLS     INTEGRATE MAP LEVELS & RETURN INDICES
C   PRGMMR: M. BARTH         ORG: FSL        DATE: 02-04-23
C
C ABSTRACT: CREATE THE INTEGRATED PROFILES THAT COMBINE ALL WIND MODES 
C   AND TEMPERATURE PROFILES INTO A SINGLE PROFILE ASCENDING IN HEIGHT.
C
C PROGRAM HISTORY LOG:
C   02-04-23  M. BARTH    ORIGINAL VERSION  
C   06-09-01  L. BENJAMIN ADDED DYNAMIC SETTING NUMBER OF TEMP LEVELS.
C                         MXTLVL IS MAX POSSIBLE AND NMTLVL IS NUMBER
C                         IN NETCDF FILE.
C   08-06-20  L. BENJAMIN CHANGED MADISCON TO INTQC1CON2 AND 
C                         MAPCON.  CHANGED MRDNUM CALL TO RDNUM. ADDED
C                         TEMP PROCESSING MEMEORY FOR WRITE CALLS.
C
C USAGE:    CALL MAPLVLS (NRECW,NRECT,LSTRCQ,NRECI,STATUS)
C
C   INPUT ARGUMENT LIST:
C
C     NRECW    - I       NUMBER OF INPUT WIND RECORDS
C     NRECT    - I       NUMBER OF INPUT TEMP RECORDS
C     LSTRCQ   - I       NUMBER OF OUTPUT RECORDS ALREADY IN QC FILE
C     NRECI    - I       NUMBER OF INTEGRATED INPUT RECORDS
C
C   OUTPUT ARGUMENT LIST:
C     STATUS   - I       SUCCESS_P OR FAILURE_P IF READ/WRITE PROBLEM
C
C   INPUT FILES:
C     WIND FILE- NETCDF FILE OPEN ON NCIDW
C     TEMP FILE- NETCDF FILE OPEN ON NCIDT
C
C   OUTPUT FILES:
C     QC FILE  - NETCDF FILE OPEN ON NCIDQ
C
C SUBPROGRAMS CALLED:
C     - MADIS LIBRARY:  M_ASCSORT,M_RSWAP
C     - UNIQUE:         RDNUM,MWTNUM
C
C REMARKS:
C
C   OUTPUTS LINDEX VARIABLE IN MAPCOM.
C
C ATTRIBUTES:
C   LANGUAGE: FORTRAN-77 + EXTENSIONS
C   MACHINE:  UNIX
C$$$


      SUBROUTINE MAPLVLS (NRECW,NRECT,LSTRCQ,NRECI,STATUS)


C     ------------------------------------------------------------------
C     INCLUDES AND COMMONS
C     ------------------------------------------------------------------
      INCLUDE 'IMPLICIT'
      INCLUDE 'INTQC1CON2'
      INCLUDE 'MAPCON'
      INCLUDE 'STDOUT'
      INCLUDE 'MAPCOM'


C     ------------------------------------------------------------------
C     ARGUMENTS
C     ------------------------------------------------------------------
      INTEGER NRECW,NRECT,LSTRCQ,NRECI,STATUS


C     ------------------------------------------------------------------
C     LOCAL
C     ------------------------------------------------------------------
      INTEGER I,J,K,N,RCODE
      REAL LVLS(MXMAPLEV_P,MXMAP_P),WLVLS(MXMODE,MXWLVL,MXMAP_P)
      REAL TLVLS(MXTLVL*MXMAP_P),WRM(MXMAPLEV_P,MXMAP_P)
      REAL SWAP_TMP(MXMAPLEV_P)
      REAL NWMODES(MXMAP_P),NTMODES(MXMAP_P)
      INTEGER NX
C     *****************************************************************
C     ******************* BEGIN OF MAPLVLS ****************************
C     *****************************************************************

      STATUS = FAILURE_P

C     ------------------------------------------------------------------
C     FILL IN MISSING FLAGS IN ALL POSSIBLE LEVELS.
C     ------------------------------------------------------------------
      DO I = 1, NRECI
         DO J = 1, MXMAPLEV_P
            LVLS(J,I) = MISS_P
            WRM(J,I) = MISS_P
         ENDDO
         NTMODES(I) = MISS_P
      ENDDO

C     ------------------------------------------------------------------
C     GET THE WIND RECORDS FIRST (IF ANY).  PUT THOSE INTO THE 
C     INTEGRATED LVLS ARRAY.  THEN GET THE TEMPERATURE RECORDS (IF ANY),
C     AND TACK THOSE ON THE END.  WHILE WE'RE FILLING IN THE LEVELS,
C     FILL IN THE "WIND-RASS-MODE" VARIABLE (1-MXMODE = WIND MODE, 
C     MXMODE + 1 = RASS).  ALSO NOTE THAT FOR EACH WIND RECORD WE'LL
C     FILL IN ZERO RASS MODES IN USE.  THAT WAY, WHEN WE'RE ALL DONE
C     EACH RECORD WILL HAVE EITHER 0 OR 1 RASS MODES (RATHER THAN HAVING
C     FILL VALUES OR 1'S).
C     ------------------------------------------------------------------
      IF(NRECW.NE.0)THEN

         CALL RDNUM('levels',NCIDW,RECDIMW,1,NRECW,MISS_P,WLVLS,
     1        NX,RCODE)
         IF(RCODE.NE.SUCCESS_P)GO TO 900
         CALL RDNUM('numModesUsed',NCIDW,RECDIMW,1,NRECW,MISS_P,
     1               NWMODES,NX,RCODE)
         IF(RCODE.NE.SUCCESS_P)GO TO 900

         DO I = 1, NRECW
            NTMODES(I) = 0
            N = 0
            DO K = 1, NWMODES(I)
               DO J = 1, MXWLVL
                  N = N + 1
                  LVLS(N,I) = WLVLS(K,J,I)
                  IF(LVLS(N,I).NE.MISS_P)WRM(N,I) = K
               ENDDO
            ENDDO
         ENDDO

      ENDIF

      IF(NRECT.NE.0)THEN

         CALL RDNUM('levels',NCIDT,RECDIMT,1,NRECT,MISS_P,TLVLS,
     1        NX,RCODE)
         IF(RCODE.NE.SUCCESS_P)GO TO 900
         DO I = 1, NRECT
            NTMODES(TREC(I)) = 1
            N = MXMODE * MXWLVL
            DO J = 1, NMTLVL
               N = N + 1
               LVLS(N,TREC(I)) = TLVLS((I-1)*NMTLVL+J)
               IF(LVLS(N,TREC(I)).NE.MISS_P)WRM(N,TREC(I)) = MXMODE + 1
            ENDDO
         ENDDO

      ENDIF

C     ------------------------------------------------------------------
C     DO AN ASCENDING SORT BY HEIGHT, AND SAVE THE ORIGINAL INDICES
C     IN COMMON'S LINDEX VARIABLE FOR LATER USE.  NOTE THAT THIS 
C     REQUIRES MISS_P TO BE > THE LARGEST GENUINE DATA VALUE -- LUCKILY, 
C     THAT'S TRUE.  ALSO, MOVE THE WRM ARRAY AROUND TO MATCH THE
C     ASCENDING LEVEL ORDER.
C     ------------------------------------------------------------------
      DO I = 1, NRECI
         CALL M_ASCSORT(LVLS(1,I),LINDEX(1,I),MXMAPLEV_P)
         CALL M_RSWAP(MXMAPLEV_P,LINDEX(1,I),WRM(1,I),SWAP_TMP)
      ENDDO

C     ------------------------------------------------------------------
C     OUTPUT THE INTEGRATED LEVELS, "WIND-RASS-MODE", AND NUMBER OF
C     RASS MODES VARIABLES.
C     ------------------------------------------------------------------
      CALL MWTNUM('levels',NCIDQ,RECDIMQ,LSTRCQ+1,NRECI,MISS_P,LVLS,
     1            RCODE,MXMAP,
     2 RMAPDATA,IMAPDATA,I2MAPDATA,BMAPDATA)
      IF(RCODE.NE.SUCCESS_P)GO TO 900

      CALL MWTNUM('windRASSMode',NCIDQ,RECDIMQ,LSTRCQ+1,NRECI,MISS_P,
     1            WRM,RCODE,MXMAP,
     2 RMAPDATA,IMAPDATA,I2MAPDATA,BMAPDATA)

      CALL MWTNUM('numRASSModesUsed',NCIDQ,RECDIMQ,LSTRCQ+1,NRECI,
     1            MISS_P,NTMODES,STATUS,MXMAP,
     2 RMAPDATA,IMAPDATA,I2MAPDATA,BMAPDATA)

 900  RETURN
      END

