C$$$  SUBPROGRAM DOCUMENTATION BLOCK
C                .      .    .                                       .
C SUBPROGRAM:    MAPSFCCSV   READ MAP SFC OBS, OUTPUT TO CSV FILES
C   PRGMMR: M. BARTH         ORG: FSL        DATE: 02-06-26
C
C ABSTRACT: READ SFC OBS FROM A MAP NETCDF FILE, AND OUTPUT IN LDAD
C   FORMAT TO CSV FILES (ONE FILE PER RECORD)
C
C PROGRAM HISTORY LOG:
C   02-06-26  M. BARTH    ORIGINAL VERSION 
C   02-07-16  M. BARTH    FIXED BUG IN THE DO J = 1, 8 LOOP WHERE
C                         THE CSV RECORD IS WRITTEN TO THE LINE
C                         VARIABLE.  WE WERE LOOKING AT I TO DECIDE 
C                         WHICH FORMAT TO USE, BUT SHOULD HAVE BEEN 
C                         LOOKING AT I.
C   07-12-04  L. BENJAMIN ADDED MORE SURFACE VARIABLES, NOW 11
C                         INCREASE VNAME LENGTH AND SIZE
C   08-06-20  L. BENJAMIN CHANGED MADISCON TO INTQC1CON2 AND 
C                         MAPCON.  CHANGED MRDNUM CALL TO RDNUM.
C
C USAGE:    CALL MAPSFCCSV (ATIME,NRECS,STAID,DTIME,STATUS)
C
C   INPUT ARGUMENT LIST:
C     ATIME    - C       YYJJJHHMM OF HOUR TO BE PROCESSED
C     NRECS    - I       NUMBER OF OB RECORDS
C     STAID    - CA      STATION NAMES
C     DTIME    - DA      STORED OB TIMES (NOT I4TIMES)
C
C   OUTPUT ARGUMENT LIST:
C
C     STATUS   - I       SUCCESS_P OR FAILURE_P IF OUTPUT FILE PROBLEM
C
C   INPUT FILES:
C     QC FILE  - QC_DIR/FILENAME WHERE FILENAME IS CREATED FROM ATIME
C
C   OUTPUT FILES:
C     CSV FILES- CSV FILES OF SFC DATA FOR LDAD (FILENAMES ARE CREATED
C                IN SUBROUTINE THAT WRITES THE FILES)
C
C SUBPROGRAMS CALLED:
C     LIBRARY: - NETCDF:
C                NCCLOS
C                NCINQ
C                NCOPN
C              - MADIS LIBRARY:
C                M_CNVTTM
C                M_GETLUN
C                M_MKWFNM
C                
C     UNIQUE: RDNUM
C
C REMARKS:
C   NONE
C
C ATTRIBUTES:
C   LANGUAGE: FORTRAN-77 + EXTENSIONS
C   MACHINE:  UNIX
C$$$


      SUBROUTINE MAPSFCCSV (ATIME,NRECS,STAID,DTIME,STATUS)

C     ------------------------------------------------------------------
C     INCLUDES AND COMMONS
C     ------------------------------------------------------------------
      INCLUDE 'IMPLICIT'
      INCLUDE 'INTQC1CON2'
      INCLUDE 'MAPCON'
      INCLUDE 'NETCDFI'
      INCLUDE 'STDOUT'

C     ------------------------------------------------------------------
C     ARGUMENTS
C     ------------------------------------------------------------------
      CHARACTER*9      ATIME
      INTEGER          NRECS
      CHARACTER*6      STAID(*)
      DOUBLE PRECISION DTIME(*)
      INTEGER          STATUS

C     ------------------------------------------------------------------
C     LOCAL
C     ------------------------------------------------------------------
      INTEGER       I,J,K,LUN
      CHARACTER*13  FNAME
      CHARACTER*17  TIMSTR
      CHARACTER*122 LINE
      INTEGER       NCID,RCODE,RECDIM,NDIMS,NVARS,NGATTS
      INTEGER       YEAR,MONTH,DAY,HR,MINUTE,SEC,I4TIME
      REAL          RDATA(MXMAP_P,11)
      DOUBLE PRECISION DMISS
      CHARACTER*19  VNAME(11)
      INTEGER       NX,VNLEN(11)
      
C     -----------------------------------------------------------------
C     DATA STATEMENTS (KEEP AT BOTTOM OF DECLARATIONS)
C     -----------------------------------------------------------------
      DATA DMISS/0.0D0/
      DATA VNAME/'temperatureSfc',
     1           'windSpeedSfc',
     2           'windDirSfc',
     3           'altimeter',
     4           'pressure',
     5           'seaLevelPressure',
     6           'relHumidity',
     7           'rainRate',
     8           'soilTemperature',
     9           'soilMoisturePercent',
     1           'solarRadiation'/

      DATA VNLEN/14,12,10,9,8,16,11,8,15,19,14/


C     *****************************************************************
C     ******************* BEGIN OF MAPSFCCSV **************************
C     *****************************************************************

      STATUS = FAILURE_P
      CALL M_GETLUN(LUN,1)

C     ------------------------------------------------------------------
C     CREATE THE FILENAME FROM ATIME
C     ------------------------------------------------------------------
      WRITE(STDOUT,*)'     - CREATING QC FILENAME FROM ATIME: ',ATIME
      CALL M_MKWFNM(ATIME, FNAME, RCODE)
      IF(RCODE .NE. SUCCESS_P)RETURN

C     ------------------------------------------------------------------
C     OPEN THE QC FILE FOR THIS TIME.
C     ------------------------------------------------------------------
      WRITE(STDOUT,*)'     - OPENING QC_DIR/'//FNAME
      NCID = NCOPN('QC_DIR/'//FNAME, NCNOWRIT, RCODE)
      IF(RCODE .NE. SUCCESS_P)THEN
         WRITE(STDOUT,*)'          - ERROR OPENING FILE: ',RCODE
         RETURN
      ENDIF

C     ------------------------------------------------------------------
C     GET THE RECORD DIMENSION ID.
C     ------------------------------------------------------------------
      CALL NCINQ(NCID,NDIMS,NVARS,NGATTS,RECDIM,RCODE)
      IF(RCODE.NE.0)THEN
         WRITE(STDOUT,*)'          - NCINQ ERROR: ',RCODE
         GO TO 900
      ENDIF

C     ------------------------------------------------------------------
C     READ IN ALL THE SURFACE VARIABLES.  
C     ------------------------------------------------------------------
      DO I = 1, 11

         CALL RDNUM(VNAME(I)(1:VNLEN(I)),NCID,RECDIM,1,NRECS,MISS_P,
     1               RDATA(1,I),NX,RCODE)
         IF(RCODE.NE.0)GO TO 900

      ENDDO

C     ------------------------------------------------------------------
C     FOR EACH STATION, SEE IF IT HAS ANY NON-MISSING SURFACE OBS.
C     IF SO, THEN WE'LL OUTPUT A FILE FOR THAT STATION/TIME, WITH THE
C     TIME AND STATION IN THE FILENAME.  FOR EXAMPLE, 
C     020626120000.BBYCA IS FOR JUNE 26, 2002 12:00:00 UTC AND BBYCA IS 
C     THE STATION NAME.
C
C     ALSO NOTE THAT THE MISSING DATA FLAG IS -9999.00 -- EXACTLY THIS 
C     STRING.
C     ------------------------------------------------------------------
      DO 100 I = 1, NRECS
         
         IF(DTIME(I).NE.DMISS)THEN

            DO J = 1, 11
               IF(RDATA(I,J).NE.MISS_P)GO TO 200
            ENDDO

         ENDIF
         GO TO 100

C     ------------------------------------------------------------------
C     CONVERT UNIX TIME TO I4TIME, THEN TO INTEGER DAY,MONTH,YR, ETC.,
C     AND FINALLY CONVERT TO LDAD TIME STRING:  "YY/MM/DD HH:MM:SS".
C
C     NOTE THAT THE "YEAR" RETURNED FROM M_CNVTTM IS THE YEAR SINCE
C     1900 -- E.G., 2002 GIVES YOU 102.  ALSO NOTE THAT I4TIME DOESN'T
C     DEAL WITH SECONDS -- SO WE'LL GET THOSE DIRECTLY FROM DTIME.
C     ------------------------------------------------------------------
 200     I4TIME = DTIME(I) - 1325376000
         CALL M_CNVTTM(I4TIME,YEAR,MONTH,DAY,HR,MINUTE,SEC,RCODE)
         IF(RCODE.NE.SUCCESS_P)GO TO 100
         IF(YEAR.GE.100)YEAR = YEAR - 100
         SEC = MOD(DTIME(I),60)
         WRITE(TIMSTR,1,ERR=100)YEAR,MONTH,DAY,HR,MINUTE,SEC
 1       FORMAT(I2,'/',I2,'/',I2,' ',I2,':',I2,':',I2)
         IF(TIMSTR(1:1).EQ.' ')TIMSTR(1:1) = '0'
         IF(TIMSTR(4:4).EQ.' ')TIMSTR(4:4) = '0'
         IF(TIMSTR(7:7).EQ.' ')TIMSTR(7:7) = '0'
         IF(TIMSTR(10:10).EQ.' ')TIMSTR(10:10) = '0'
         IF(TIMSTR(13:13).EQ.' ')TIMSTR(13:13) = '0'
         IF(TIMSTR(16:16).EQ.' ')TIMSTR(16:16) = '0'

         FNAME = TIMSTR(1:2)//TIMSTR(4:5)//TIMSTR(7:8)//TIMSTR(10:11)//
     1           TIMSTR(13:14)//TIMSTR(16:17)
         OPEN(UNIT=LUN,FILE='CSV_DIR/'//FNAME(1:12)//'.'//STAID(I),
     1        STATUS='UNKNOWN',ERR=100)
         LINE(1:23) = STAID(I)(1:5)//','//TIMSTR

         DO J = 1, 11

            K = (J - 1) * 9 + 24
            LINE(K:K) = ','
            IF(RDATA(I,J).EQ.MISS_P)THEN
               
               LINE(K+1:K+8) = '-9999.00'

            ELSE IF(J.LE.3.OR.J.EQ.7)THEN

               WRITE(LINE(K+1:K+8),2)RDATA(I,J)
 2             FORMAT(F8.4)

            ELSE IF(J.EQ.8)THEN

               WRITE(LINE(K+1:K+8),3)RDATA(I,J)
 3             FORMAT(F8.6)

            ELSE

               WRITE(LINE(K+1:K+8),4)RDATA(I,J)
 4             FORMAT(F8.1)

            ENDIF

         ENDDO

         write(*,*)'record ',i
         write(*,5)line
         WRITE(LUN,5)LINE
 5       FORMAT(A)
         CLOSE(UNIT=LUN)

 100  CONTINUE

      STATUS = SUCCESS_P

 900  CALL NCCLOS(NCID,RCODE)
      CALL M_GETLUN(LUN,2)

      RETURN
      END
