C$$$  SUBPROGRAM DOCUMENTATION BLOCK
C                .      .    .                                       .
C SUBPROGRAM:    MAPVPROC    PROCESS MAP VARS. WITH LITTLE SPECIAL HDNLG
C   PRGMMR: M. BARTH         ORG: FSL        DATE: 02-04-24
C
C ABSTRACT: HANDLE INPUT, INTEGRATION OF RECORDS/LEVELS, AND OUTPUT OF
C   MAP NETCDF VARIABLES THAT DON'T REQUIRE (MUCH) SPECIAL HANDLING
C
C PROGRAM HISTORY LOG:
C   02-04-24  M. BARTH    ORIGINAL VERSION  
C   06-09-01  L. BENJAMIN ADDED DYNAMIC SETTING NUMBER OF TEMP LEVELS.
C                         MXTLVL IS MAX POSSIBLE AND NMTLVL IS NUMBER
C                         IN NETCDF FILE.
C   08-06-20  L. BENJAMIN CHANGED MADISCON TO INTQC1CON2 AND 
C                         MAPCON.  CHANGE MRDBDL CALL TO LOCAL 
C                         RDDBL.
C
C USAGE:    CALL MAPVPROC (NRECW,NRECT,LSTRCQ,NRECI,WTVAR,VTCHAR,
C                          VNAMEI,VNAMEO,LDIM,BDIM,STATUS)
C
C   INPUT ARGUMENT LIST:
C
C     NRECW    - I       NUMBER OF INPUT WIND RECORDS
C     NRECT    - I       NUMBER OF INPUT TEMP RECORDS
C     LSTRCQ   - I       NUMBER OF OUTPUT RECORDS ALREADY IN QC FILE
C     NRECI    - I       NUMBER OF INTEGRATED INPUT RECORDS
C     WTVAR    - C       'W' - WIND, 'T' - TEMP, 'B' - BOTH
C     VTCHAR   - C       NETCDF VARIABLE TYPE - IN A SINGLE CHARACTER
C     VNAMEI   - C       VARIABLE NAME IN INPUT FILE
C     VNAMEO   - C       VARIABLE NAME IN OUTPUT FILE
C     LDIM     - C       'L' - HAS LEVEL DIMENSION, ' ' - DOESN'T
C     BDIM     - C       'B' - HAS BEAM DIMENSION, ' ' - DOESN'T
C
C   OUTPUT ARGUMENT LIST:
C     STATUS   - I       SUCCESS_P OR FAILURE_P IF READ/WRITE PROBLEM
C
C   INPUT FILES:
C     WIND FILE- NETCDF FILE OPEN ON NCIDW
C     TEMP FILE- NETCDF FILE OPEN ON NCIDT
C
C   OUTPUT FILES:
C     QC FILE  - NETCDF FILE OPEN ON NCIDQ
C
C SUBPROGRAMS CALLED:
C     - MADIS LIBRARY:  MWTNUM,MRDCHR,MWTCHR
C     - UNIQUE: RDNUM
C
C REMARKS:
C
C   HERE ARE SOME IMPORTANT ASSUMPTIONS ABOUT "BOTH" VARIABLES:
C
C   - THE WIND VARIABLES GET PROCESSED FIRST.  
C   - THE WIND & TEMP VARIABLES MUST HAVE THE SAME DIMENSIONS (EXCEPT
C     FOR THE RECORD DIMENSION, OF COURSE).
C   - IF A GIVEN RECORD EXISTS FOR BOTH WIND & TEMP, THE TEMP ONE
C     IS THE ONE THAT WILL GET PROCESSED LAST, THEREFORE THE DATA
C     BETTER BE THE SAME AS IN THE WIND RECORD PROCESSED FIRST...
C
C   THERE ARE TWO "SPECIAL HANDLING" CASES:  vertCorrectionFlag AND
C   RASSwComponent
C
C ATTRIBUTES:
C   LANGUAGE: FORTRAN-77 + EXTENSIONS
C   MACHINE:  UNIX
C$$$


      SUBROUTINE MAPVPROC (NRECW,NRECT,LSTRCQ,NRECI,WTVAR,VTCHAR,VNAMEI,
     1                     VNAMEO,LDIM,BDIM,STATUS)


C     ------------------------------------------------------------------
C     INCLUDES AND COMMONS
C     ------------------------------------------------------------------
      INCLUDE 'IMPLICIT'
      INCLUDE 'NETCDFI'
      INCLUDE 'INTQC1CON2'
      INCLUDE 'MAPCON'
      INCLUDE 'STDOUT'
      INCLUDE 'MAPCOM'

C     ------------------------------------------------------------------
C     ARGUMENTS
C     ------------------------------------------------------------------
      INTEGER NRECW,NRECT,LSTRCQ,NRECI,STATUS
      CHARACTER*1 WTVAR,VTCHAR,LDIM,BDIM
      CHARACTER*(*) VNAMEI,VNAMEO

C     ------------------------------------------------------------------
C     LOCAL
C     ------------------------------------------------------------------
      INTEGER I,J,K,NW,NT,N,RCODE,VTYPE,IOSTAT,MODE,LEVEL

      CHARACTER*1 CDATA(MAX(MODENAMELEN,BEAMNAMELEN)*MXMAP_P)

      REAL DOUT(MXMAPLEV_P*MXMAP_P*MXBEAM)
      REAL DOUTL(MXMAPLEV_P,MXMAP_P)
      REAL DOUTLB(MXMAPLEV_P,MXBEAM,MXMAP_P)
      EQUIVALENCE (DOUT,DOUTL,DOUTLB)

      REAL DINW(MXWLVL*MXMODE*MXBEAM*MXMAP_P)
      REAL DINWL(MXMODE,MXWLVL,MXMAP_P)
      REAL DINWLB(MXMODE,MXWLVL,MXBEAM,MXMAP_P)
      EQUIVALENCE (DINW,DINWL,DINWLB)

      REAL DINT(MXTLVL*MXMAP_P)


C     *****************************************************************
C     ******************* BEGIN OF MAPVPROC ***************************
C     *****************************************************************

      STATUS = FAILURE_P

C     ------------------------------------------------------------------
C     IF THE VARIABLE BEING PROCESSED IS EXCLUSIVELY WIND OR TEMP, AND
C     THERE ARE NO RECORDS OF THAT TYPE, WE'RE DONE.
C     ------------------------------------------------------------------
      IF((WTVAR.EQ.'W'.AND.NRECW.EQ.0).OR.
     1   (WTVAR.EQ.'T'.AND.NRECT.EQ.0))THEN

         STATUS = SUCCESS_P
         GO TO 900

      ENDIF

C     ------------------------------------------------------------------
C     CONVERT THE VARIABLE TYPE CHARACTER TO THE NETCDF INTEGER CODE.
C     ALL WE NEED THIS FOR IS TO DECIDE BETWEEN CHARACTER & NUMERIC.
C     ------------------------------------------------------------------
      READ(VTCHAR,1,IOSTAT=IOSTAT)VTYPE
 1    FORMAT(I1)
      IF(IOSTAT.NE.0)THEN
         WRITE(STDOUT,*)'          - FAILED TO READ VTYPE FOR ',WTVAR,
     1                  ' ',VNAMEI
         GO TO 900
      ENDIF

C     ------------------------------------------------------------------
C     IF IT'S CHARACTER, WE KNOW A PRIORI THAT IT'S IN THE WIND FILE
C     ONLY, AND THERE'S NO LEVELS OR BEAMS TO SCREW AROUND WITH.
C     ------------------------------------------------------------------
      IF(VTYPE.EQ.NCCHAR)THEN

         IF(NRECW.NE.0)THEN

            CALL MRDCHR(VNAMEI,NCIDW,RECDIMW,1,NRECW,CDATA,I,RCODE)
            IF(RCODE.NE.0)GO TO 900
            CALL MWTCHR(VNAMEO,NCIDQ,RECDIMQ,LSTRCQ+1,NRECW,CDATA,RCODE)
            IF(RCODE.NE.0)GO TO 900

         ENDIF

         STATUS = SUCCESS_P
         GO TO 900

      ENDIF

C     ------------------------------------------------------------------
C     FILL IN MISSING FLAGS IN THE OUTPUT DATA ARRAY.
C     ------------------------------------------------------------------
      DO I = 1, MXMAPLEV_P * MXMAP_P * MXBEAM
         DOUT(I) = MISS_P
      ENDDO

C     ------------------------------------------------------------------
C     READ IN THE NUMERIC WIND VARIABLE INTO AN INPUT ARRAY.
C     - NW IS THE NUMBER OF ELEMENTS FOR A SINGLE RECORD.
C     ------------------------------------------------------------------
      IF(NRECW.NE.0.AND.(WTVAR.EQ.'W'.OR.WTVAR.EQ.'B'))THEN

         CALL RDNUM(VNAMEI,NCIDW,RECDIMW,1,NRECW,MISS_P,DINW,N,RCODE)
         IF(RCODE.NE.0)GO TO 900
         NW = N/NRECW

      ENDIF

C     ------------------------------------------------------------------
C     READ IN THE NUMERIC TEMP VARIABLE INTO AN INPUT ARRAY.
C     - NT IS THE NUMBER OF ELEMENTS FOR A SINGLE RECORD.
C     ------------------------------------------------------------------
      IF(NRECT.NE.0.AND.(WTVAR.EQ.'T'.OR.WTVAR.EQ.'B'))THEN

         CALL RDNUM(VNAMEI,NCIDT,RECDIMT,1,NRECT,MISS_P,DINT,N,RCODE)
         IF(RCODE.NE.0)GO TO 900
         NT = N/NRECT

      ENDIF

C     ------------------------------------------------------------------
C     COPY TO THE OUTPUT ARRAY.
C     ------------------------------------------------------------------
      IF(LDIM.NE.'L')THEN

         IF(WTVAR.EQ.'W'.OR.WTVAR.EQ.'B')THEN

C     ------------------------------------------------------------------
C     ONE SPECIAL-HANDLING CASE IN THIS SECTION:
C
C     THE vertCorrectionFlag VARIABLE HAS THESE INPUT/OUTPUT VALUES:
C
C     INPUT             OUTPUT
C     -----             ------
C       0                 0   - OFF
C       1                 1   - ON
C       MISS_P            2   - UNKNOWN
C     ------------------------------------------------------------------
            IF(VNAMEI.NE.'vertCorrectionFlag')THEN

               DO I = 1, NRECW * NW
                  DOUT(I) = DINW(I)
               ENDDO

            ELSE

               DO I = 1, NRECW * NW
                  IF(DINW(I).NE.MISS_P)THEN
                     DOUT(I) = DINW(I)
                  ELSE
                     DOUT(I) = 2
                  ENDIF
               ENDDO

            ENDIF

         ENDIF

         IF(WTVAR.EQ.'T'.OR.WTVAR.EQ.'B')THEN

            DO I = 1, NRECT

               DO K = 1, NT
                  DOUT(TREC(I)-1*NT+K) = DINT((I-1)*NT+K)
               ENDDO

            ENDDO

         ENDIF

      ELSE IF(BDIM.NE.'B')THEN

C     ------------------------------------------------------------------
C     WITH LEVEL OR LEVEL/BEAM DIMENSIONED VARIABLES, MOVE THE 
C     LEVELS TO WHERE THEY BELONG IN THE INTEGRATED OUTPUT ORDER.
C     ------------------------------------------------------------------
         IF(WTVAR.EQ.'W'.OR.WTVAR.EQ.'B')THEN

            DO I = 1, NRECW

               DO J = 1, MXMAPLEV_P
               
                  IF(LINDEX(J,I).LE.MXMODE*MXWLVL)THEN

                     MODE = (LINDEX(J,I) - 1)/MXWLVL + 1
                     LEVEL = LINDEX(J,I) - (MODE - 1) * MXWLVL
                     DOUTL(J,I) = DINWL(MODE,LEVEL,I)

                  ENDIF

               ENDDO

            ENDDO

         ENDIF

         IF(WTVAR.EQ.'T'.OR.WTVAR.EQ.'B')THEN

C     ------------------------------------------------------------------
C     ONE SPECIAL-HANDLING CASE IN THIS SECTION:
C
C     THE RASS W COMPONENT HAS TO BE CONVERTED FROM M/S TO KTS.
C     ------------------------------------------------------------------
            IF(VNAMEO.NE.'RASSwComponent')THEN

               DO I = 1, NRECT

                  DO J = 1, MXMAPLEV_P
               
                     IF(LINDEX(J,TREC(I)).GT.MXMODE*MXWLVL.AND.
c$$$     1                  LINDEX(J,TREC(I)).LE.MXMODE*MXWLVL+MXTLVL)THEN
     1                  LINDEX(J,TREC(I)).LE.MXMODE*MXWLVL+NMTLVL)THEN

                        LEVEL = LINDEX(J,TREC(I)) - (MXMODE * MXWLVL)
c$$$                        DOUTL(J,TREC(I)) = DINTL(LEVEL,I)
                        DOUTL(J,TREC(I)) = DINT((I-1)*NMTLVL+LEVEL)
                     ENDIF
                  
                  ENDDO

               ENDDO

            ELSE

               DO I = 1, NRECT

                  DO J = 1, MXMAPLEV_P
               
                     IF(LINDEX(J,TREC(I)).GT.MXMODE*MXWLVL.AND.
c$$$     1                  LINDEX(J,TREC(I)).LE.MXMODE*MXWLVL+MXTLVL)THEN
     1                  LINDEX(J,TREC(I)).LE.MXMODE*MXWLVL+NMTLVL)THEN

                        LEVEL = LINDEX(J,TREC(I)) - (MXMODE * MXWLVL)

c$$$                        IF(DINTL(LEVEL,I).NE.MISS_P)
                        IF(DINT((I-1)*NMTLVL+LEVEL).NE.MISS_P)
     1                     DOUTL(J,TREC(I)) = DINT((I-1)*NMTLVL+LEVEL) * 
     2                                        1.943861286

                     ENDIF
                  
                  ENDDO

               ENDDO

            ENDIF

         ENDIF

      ELSE IF(BDIM.EQ.'B')THEN

C     ------------------------------------------------------------------
C     ONLY WINDS HAVE A BEAM DIMENSION.
C     ------------------------------------------------------------------
         DO I = 1, NRECW

            DO J = 1, MXMAPLEV_P
               
               IF(LINDEX(J,I).LE.MXMODE*MXWLVL)THEN

                  MODE = (LINDEX(J,I) - 1)/MXWLVL + 1
                  LEVEL = LINDEX(J,I) - (MODE - 1) * MXWLVL

                  DO K = 1, MXBEAM
                     DOUTLB(J,K,I) = DINWLB(MODE,LEVEL,K,I)
                  ENDDO

               ENDIF

            ENDDO

         ENDDO

      ENDIF

C     ------------------------------------------------------------------
C     PHEW... WRITE THE DARNED THING.
C     ------------------------------------------------------------------
      CALL MWTNUM(VNAMEO,NCIDQ,RECDIMQ,LSTRCQ+1,NRECI,MISS_P,DOUT,
     1            STATUS,MXMAP,
     2 RMAPDATA,IMAPDATA,I2MAPDATA,BMAPDATA)

 900  RETURN
      END
