C$$$  SUBPROGRAM DOCUMENTATION BLOCK
C                .      .    .                                       .
C SUBPROGRAM:    MAPWINDS    PROCESS MAP WINDS AND QC
C   PRGMMR: M. BARTH         ORG: FSL        DATE: 02-04-25
C
C ABSTRACT: HANDLE INPUT, INTEGRATION OF LEVELS, QC STRUCTURE
C   TRANSLATION, AND OUTPUT OF MAP WIND VARIABLES
C
C PROGRAM HISTORY LOG:
C   02-04-25  M. BARTH    ORIGINAL VERSION  
C   08-06-20  L. BENJAMIN CHANGED MADISCON TO INTQC1CON2 AND 
C                         MAPCON.  CHANGED MRDNUM CALL TO RDNUM. ADDED
C                         TEMP PROCESSING MEMEORY FOR WRITE CALLS.
C
C USAGE:    CALL MAPWINDS (NRECW,NRECI,LSTRCQ,STATUS)
C
C   INPUT ARGUMENT LIST:
C
C     NRECW    - I       NUMBER OF INPUT WIND RECORDS
C     NRECI    - I       NUMBER OF INTEGRATED INPUT RECORDS
C     LSTRCQ   - I       NUMBER OF OUTPUT RECORDS ALREADY IN QC FILE
C
C   OUTPUT ARGUMENT LIST:
C     STATUS   - I       SUCCESS_P OR FAILURE_P IF READ/WRITE PROBLEM
C
C   INPUT FILE:
C     WIND FILE- NETCDF FILE OPEN ON NCIDW
C
C   OUTPUT FILES:
C     QC FILE  - NETCDF FILE OPEN ON NCIDQ
C
C SUBPROGRAMS CALLED:
C     - MADIS LIBRARY:  M_DDFF
C     - UNIQUE:         RDNUM,QCTRANS,MWTCHR,MWTNUM
C
C REMARKS:
C
C   THE LOGIC HEREIN RELIES ON THE ASSUMPTION THAT WIND RECORDS
C   ARE PROCESSED BEFORE TEMPERATURE RECORDS.  
C
C ATTRIBUTES:
C   LANGUAGE: FORTRAN-77 + EXTENSIONS
C   MACHINE:  UNIX
C$$$


      SUBROUTINE MAPWINDS (NRECW,NRECI,LSTRCQ,STATUS)


C     ------------------------------------------------------------------
C     INCLUDES AND COMMONS
C     ------------------------------------------------------------------
      INCLUDE 'IMPLICIT'
      INCLUDE 'NETCDFI'
      INCLUDE 'INTQC1CON2'
      INCLUDE 'MAPCON'
      INCLUDE 'STDOUT'
      INCLUDE 'MAPCOM'

C     ------------------------------------------------------------------
C     ARGUMENTS
C     ------------------------------------------------------------------
      INTEGER NRECW,NRECI,LSTRCQ,STATUS

C     ------------------------------------------------------------------
C     LOCAL
C     ------------------------------------------------------------------
      INTEGER I,J,RCODE,MODE,LEVEL

      REAL U(MXMAPLEV_P,MXMAP_P)
      REAL V(MXMAPLEV_P,MXMAP_P)
      REAL W(MXMAPLEV_P,MXMAP_P)
      REAL SPD(MXMAPLEV_P,MXMAP_P)
      REAL DIR(MXMAPLEV_P,MXMAP_P)
      REAL UIN(MXMODE,MXWLVL,MXMAP_P)
      REAL VIN(MXMODE,MXWLVL,MXMAP_P)
      REAL WIN(MXMODE,MXWLVL,MXMAP_P)
      REAL QCRIN(MXMODE,MXWLVL,MXMAP_P)
      REAL QCAIN(MXMODE,MXWLVL,MXMAP_P)
      REAL QCAPP(MXMAPLEV_P,MXMAP_P)
      REAL QCRES(MXMAPLEV_P,MXMAP_P)
      REAL ICAPP(MXMAPLEV_P,MXMAP_P)
      REAL ICRES(MXMAPLEV_P,MXMAP_P)
      CHARACTER*1 QCDESC(MXMAPLEV_P,MXMAP_P)
      REAL RSAWINDQC(MXMAPLEV_P,MXMAP_P)
      REAL RSAWQC(MXMAPLEV_P,MXMAP_P)
      INTEGER RDCNT

C     *****************************************************************
C     ******************* BEGIN OF MAPWINDS ***************************
C     *****************************************************************

      STATUS = FAILURE_P

C     ------------------------------------------------------------------
C     FILL IN MISSING FLAGS IN THE OUTPUT DATA ARRAYS.
C     ------------------------------------------------------------------
      DO I = 1, MXMAP_P 
         DO J = 1, MXMAPLEV_P
            U(J,I) = MISS_P
            V(J,I) = MISS_P
            W(J,I) = MISS_P
            SPD(J,I) = MISS_P
            DIR(J,I) = MISS_P
            QCAPP(J,I) = 0
            QCRES(J,I) = 0
            ICAPP(J,I) = 0
            ICRES(J,I) = 0
            QCDESC(J,I) = 'Z'
            RSAWINDQC(J,I) = MISS_P
            RSAWQC(J,I) = MISS_P
         ENDDO
      ENDDO

C     ------------------------------------------------------------------
C     IF THERE AREN'T ANY WIND RECORDS TO ACTUALLY PROCESS, SKIP DOWN
C     TO THE SECTION WHERE WE WRITE THE QC STRUCTURES TO THE FILE.  THE
C     ONLY REASON WE HAVE TO BE IN THIS ROUTINE IS TO MAKE SURE THAT
C     THE QC STRUCTURES WRITTEN TO THE FILE PROPERLY INDICATE "NO DATA".
C     (THIS CAN'T BE HANDLED WITH FILL VALUES.)
C     ------------------------------------------------------------------
      IF(NRECW.EQ.0)GO TO 800

C     ------------------------------------------------------------------
C     READ IN THE U/V/W COMPONENTS AND PROVIDER QC RES/APP WORDS.
C     ------------------------------------------------------------------
      CALL RDNUM('uComponent',NCIDW,RECDIMW,1,NRECW,MISS_P,UIN,RDCNT,
     1     RCODE)
      IF(RCODE.NE.0)GO TO 900
      CALL RDNUM('vComponent',NCIDW,RECDIMW,1,NRECW,MISS_P,VIN,RDCNT,
     1     RCODE)
      IF(RCODE.NE.0)GO TO 900
      CALL RDNUM('wComponent',NCIDW,RECDIMW,1,NRECW,MISS_P,WIN,RDCNT,
     1     RCODE)
      IF(RCODE.NE.0)GO TO 900
      CALL RDNUM('uvQualityCode',NCIDW,RECDIMW,1,NRECW,MISS_P,QCRIN,
     1     RDCNT,RCODE)
      IF(RCODE.NE.0)GO TO 900
      CALL RDNUM('uvQualityApplied',NCIDW,RECDIMW,1,NRECW,MISS_P,
     1     QCAIN,RDCNT,RCODE)
      IF(RCODE.NE.0)GO TO 900

C     ------------------------------------------------------------------
C     MOVE THE LEVELS TO WHERE THEY BELONG IN THE INTEGRATED OUTPUT 
C     ORDER.  CONVERT U/V/W FROM M/S TO KNOTS.  CREATE SPD/DIR.  
C     TRANSLATE THE QC STRUCTURES.  NOTE THAT RSA QC VALUES ARE 0=GOOD,
C     1=BAD, AND THAT WE DON'T HAVE QC ON W SO WE'LL JUST CALL IT GOOD
C     IF WE HAVE IT.
C     ------------------------------------------------------------------
      DO I = 1, NRECW

         DO J = 1, MXMAPLEV_P
               
            IF(LINDEX(J,I).LE.MXMODE*MXWLVL)THEN

               MODE = (LINDEX(J,I) - 1)/MXWLVL + 1
               LEVEL = LINDEX(J,I) - (MODE - 1) * MXWLVL
               IF(UIN(MODE,LEVEL,I).NE.MISS_P.AND.
     1            VIN(MODE,LEVEL,I).NE.MISS_P)THEN

                  U(J,I) = UIN(MODE,LEVEL,I) * 1.943861286
                  V(J,I) = VIN(MODE,LEVEL,I) * 1.943861286
                  CALL M_DDFF(U(J,I),V(J,I),MISS_P,DIR(J,I),SPD(J,I))
                  CALL QCTRANS(1,QCAIN(MODE,LEVEL,I),
     1                         QCRIN(MODE,LEVEL,I),QCAPP(J,I),
     2                         QCRES(J,I),ICAPP(J,I),ICRES(J,I),
     3                         QCDESC(J,I),RSAWINDQC(J,I))

               ENDIF

               IF(WIN(MODE,LEVEL,I).NE.MISS_P)THEN

                  W(J,I) = WIN(MODE,LEVEL,I) * 1.943861286
                  RSAWQC(J,I) = 0
               ENDIF

            ENDIF

         ENDDO

      ENDDO

C     ------------------------------------------------------------------
C     WRITE THE DATA VARIABLES WE CREATED.
C     ------------------------------------------------------------------
      CALL MWTNUM('uComponent',NCIDQ,RECDIMQ,LSTRCQ+1,NRECW,MISS_P,U,
     1            RCODE,MXMAP,
     2 RMAPDATA,IMAPDATA,I2MAPDATA,BMAPDATA)
      IF(RCODE.NE.SUCCESS_P)GO TO 900
      CALL MWTNUM('vComponent',NCIDQ,RECDIMQ,LSTRCQ+1,NRECW,MISS_P,V,
     1            RCODE,MXMAP,
     2 RMAPDATA,IMAPDATA,I2MAPDATA,BMAPDATA)
      IF(RCODE.NE.SUCCESS_P)GO TO 900
      CALL MWTNUM('wComponent',NCIDQ,RECDIMQ,LSTRCQ+1,NRECW,MISS_P,W,
     1            RCODE,MXMAP,
     2 RMAPDATA,IMAPDATA,I2MAPDATA,BMAPDATA)
      IF(RCODE.NE.SUCCESS_P)GO TO 900
      CALL MWTNUM('windSpeed',NCIDQ,RECDIMQ,LSTRCQ+1,NRECW,MISS_P,SPD,
     1            RCODE,MXMAP,
     2 RMAPDATA,IMAPDATA,I2MAPDATA,BMAPDATA)
      IF(RCODE.NE.SUCCESS_P)GO TO 900
      CALL MWTNUM('windDir',NCIDQ,RECDIMQ,LSTRCQ+1,NRECW,MISS_P,DIR,
     1            RCODE,MXMAP,
     2 RMAPDATA,IMAPDATA,I2MAPDATA,BMAPDATA)
      IF(RCODE.NE.SUCCESS_P)GO TO 900
      
C     ------------------------------------------------------------------
C     WRITE THE QC VARIABLES -- N IS THE NUMBER OF RECORDS TO WRITE.
C     ------------------------------------------------------------------
 800  CALL MWTCHR('uComponentDD',NCIDQ,RECDIMQ,LSTRCQ+1,NRECI,QCDESC,
     1             RCODE)
      IF(RCODE.NE.0)GO TO 900
      CALL MWTNUM('uComponentQCA',NCIDQ,RECDIMQ,LSTRCQ+1,NRECI,MISS_P,
     1            QCAPP,RCODE,MXMAP,
     2 RMAPDATA,IMAPDATA,I2MAPDATA,BMAPDATA)
      IF(RCODE.NE.SUCCESS_P)GO TO 900
      CALL MWTNUM('uComponentQCR',NCIDQ,RECDIMQ,LSTRCQ+1,NRECI,MISS_P,
     1            QCRES,RCODE,MXMAP,
     2 RMAPDATA,IMAPDATA,I2MAPDATA,BMAPDATA)
      IF(RCODE.NE.SUCCESS_P)GO TO 900
      CALL MWTNUM('uComponentICA',NCIDQ,RECDIMQ,LSTRCQ+1,NRECI,MISS_P,
     1            ICAPP,RCODE,MXMAP,
     2 RMAPDATA,IMAPDATA,I2MAPDATA,BMAPDATA)
      IF(RCODE.NE.SUCCESS_P)GO TO 900
      CALL MWTNUM('uComponentICR',NCIDQ,RECDIMQ,LSTRCQ+1,NRECI,MISS_P,
     1            ICRES,RCODE,MXMAP,
     2 RMAPDATA,IMAPDATA,I2MAPDATA,BMAPDATA)
      IF(RCODE.NE.SUCCESS_P)GO TO 900

      CALL MWTCHR('vComponentDD',NCIDQ,RECDIMQ,LSTRCQ+1,NRECI,QCDESC,
     1             RCODE)
      IF(RCODE.NE.0)GO TO 900
      CALL MWTNUM('vComponentQCA',NCIDQ,RECDIMQ,LSTRCQ+1,NRECI,MISS_P,
     1            QCAPP,RCODE,MXMAP,
     2 RMAPDATA,IMAPDATA,I2MAPDATA,BMAPDATA)
      IF(RCODE.NE.SUCCESS_P)GO TO 900
      CALL MWTNUM('vComponentQCR',NCIDQ,RECDIMQ,LSTRCQ+1,NRECI,MISS_P,
     1            QCRES,RCODE,MXMAP,
     2 RMAPDATA,IMAPDATA,I2MAPDATA,BMAPDATA)
      IF(RCODE.NE.SUCCESS_P)GO TO 900
      CALL MWTNUM('vComponentICA',NCIDQ,RECDIMQ,LSTRCQ+1,NRECI,MISS_P,
     1            ICAPP,RCODE,MXMAP,
     2 RMAPDATA,IMAPDATA,I2MAPDATA,BMAPDATA)
      IF(RCODE.NE.SUCCESS_P)GO TO 900
      CALL MWTNUM('vComponentICR',NCIDQ,RECDIMQ,LSTRCQ+1,NRECI,MISS_P,
     1            ICRES,RCODE,MXMAP,
     2 RMAPDATA,IMAPDATA,I2MAPDATA,BMAPDATA)
      IF(RCODE.NE.SUCCESS_P)GO TO 900

      CALL MWTCHR('windSpeedDD',NCIDQ,RECDIMQ,LSTRCQ+1,NRECI,QCDESC,
     1             RCODE)
      IF(RCODE.NE.0)GO TO 900
      CALL MWTNUM('windSpeedQCA',NCIDQ,RECDIMQ,LSTRCQ+1,NRECI,MISS_P,
     1            QCAPP,RCODE,MXMAP,
     2 RMAPDATA,IMAPDATA,I2MAPDATA,BMAPDATA)
      IF(RCODE.NE.SUCCESS_P)GO TO 900
      CALL MWTNUM('windSpeedQCR',NCIDQ,RECDIMQ,LSTRCQ+1,NRECI,MISS_P,
     1            QCRES,RCODE,MXMAP,
     2 RMAPDATA,IMAPDATA,I2MAPDATA,BMAPDATA)
      IF(RCODE.NE.SUCCESS_P)GO TO 900
      CALL MWTNUM('windSpeedICA',NCIDQ,RECDIMQ,LSTRCQ+1,NRECI,MISS_P,
     1            ICAPP,RCODE,MXMAP,
     2 RMAPDATA,IMAPDATA,I2MAPDATA,BMAPDATA)
      IF(RCODE.NE.SUCCESS_P)GO TO 900
      CALL MWTNUM('windSpeedICR',NCIDQ,RECDIMQ,LSTRCQ+1,NRECI,MISS_P,
     1            ICRES,RCODE,MXMAP,
     2 RMAPDATA,IMAPDATA,I2MAPDATA,BMAPDATA)
      IF(RCODE.NE.SUCCESS_P)GO TO 900

      CALL MWTCHR('windDirDD',NCIDQ,RECDIMQ,LSTRCQ+1,NRECI,QCDESC,
     1             RCODE)
      IF(RCODE.NE.0)GO TO 900
      CALL MWTNUM('windDirQCA',NCIDQ,RECDIMQ,LSTRCQ+1,NRECI,MISS_P,
     1            QCAPP,RCODE,MXMAP,
     2 RMAPDATA,IMAPDATA,I2MAPDATA,BMAPDATA)
      IF(RCODE.NE.SUCCESS_P)GO TO 900
      CALL MWTNUM('windDirQCR',NCIDQ,RECDIMQ,LSTRCQ+1,NRECI,MISS_P,
     1            QCRES,RCODE,MXMAP,
     2 RMAPDATA,IMAPDATA,I2MAPDATA,BMAPDATA)
      IF(RCODE.NE.SUCCESS_P)GO TO 900
      CALL MWTNUM('windDirICA',NCIDQ,RECDIMQ,LSTRCQ+1,NRECI,MISS_P,
     1            ICAPP,RCODE,MXMAP,
     2 RMAPDATA,IMAPDATA,I2MAPDATA,BMAPDATA)
      IF(RCODE.NE.SUCCESS_P)GO TO 900
      CALL MWTNUM('windDirICR',NCIDQ,RECDIMQ,LSTRCQ+1,NRECI,MISS_P,
     1            ICRES,RCODE,MXMAP,
     2 RMAPDATA,IMAPDATA,I2MAPDATA,BMAPDATA)
      IF(RCODE.NE.SUCCESS_P)GO TO 900

      CALL MWTNUM('wsQcFlag',NCIDQ,RECDIMQ,LSTRCQ+1,NRECI,MISS_P,
     1            RSAWINDQC,RCODE,MXMAP,
     2 RMAPDATA,IMAPDATA,I2MAPDATA,BMAPDATA)
      IF(RCODE.NE.SUCCESS_P)GO TO 900
      CALL MWTNUM('wdQcFlag',NCIDQ,RECDIMQ,LSTRCQ+1,NRECI,MISS_P,
     1            RSAWINDQC,RCODE,MXMAP,
     2 RMAPDATA,IMAPDATA,I2MAPDATA,BMAPDATA)
      IF(RCODE.NE.SUCCESS_P)GO TO 900
      CALL MWTNUM('uQcFlag',NCIDQ,RECDIMQ,LSTRCQ+1,NRECI,MISS_P,
     1            RSAWINDQC,RCODE,MXMAP,
     2 RMAPDATA,IMAPDATA,I2MAPDATA,BMAPDATA)
      IF(RCODE.NE.SUCCESS_P)GO TO 900
      CALL MWTNUM('vQcFlag',NCIDQ,RECDIMQ,LSTRCQ+1,NRECI,MISS_P,
     1            RSAWINDQC,RCODE,MXMAP,
     2 RMAPDATA,IMAPDATA,I2MAPDATA,BMAPDATA)
      IF(RCODE.NE.SUCCESS_P)GO TO 900
      CALL MWTNUM('wQcFlag',NCIDQ,RECDIMQ,LSTRCQ+1,NRECI,MISS_P,
     1            RSAWQC,STATUS,MXMAP,
     2 RMAPDATA,IMAPDATA,I2MAPDATA,BMAPDATA)

 900  RETURN
      END
