C$$$  SUBPROGRAM DOCUMENTATION BLOCK
C                .      .    .                                       .
C SUBPROGRAM:    MKMAPFILE   CREATE/UPDATE MAP QC NETCDF FILE
C   PRGMMR: M. BARTH         ORG: FSL        DATE: 02-04-23
C
C ABSTRACT: CREATE OR UPDATE A MAP DATASET QC NETCDF FILE FROM A 
C   PAIR OF RAW NETCDF FILES.
C
C PROGRAM HISTORY LOG:
C   02-04-23  M. BARTH    ORIGINAL VERSION  
C   06-09-06  L. BENJAMIN CHECK FOR DOUBLE PRECISION VALUES, HANDLE 
C                WITH TIME ROUTINE.  ONLY SINGLES AND MODE ONLY
C                DOUBLE PRECISION VALUES HANDLED.
C   08-06-20  L. BENJAMIN CHANGED MADISCON TO INTQC1CON2 AND 
C                         MAPCON.  ADDED TEMP PROCESSING MEMORY
C                         FOR WRITE CALLS.
C
C USAGE:    CALL MKMAPFILE (ATIME,CRES,OTIME,LSTRCQ,NRECW,NRECT,NRECI,
C                           STATUS)
C
C   INPUT ARGUMENT LIST:
C     ATIME    - C       YYJJJHHMM OF INPUT DATA TO BE PROCESSED
C     CRES     - C       TIME RESOLUTION PART OF FILENAME
C     OTIME    - C       YYJJJHHMM OF OUTPUT FILE
C
C   INPUT/OUTPUT ARGUMENT LIST:
C
C     LSTRCQ   - I       LAST RECORD IN OUTPUT QC FILE (0 FIRST TIME)
C
C   OUTPUT ARGUMENT LIST:
C     NRECW    - I       NUMBER OF INPUT WIND RECORDS
C     NRECT    - I       NUMBER OF INPUT TEMP RECORDS
C     NRECI    - I       NUMBER OF INTEGRATED INPUT RECORDS
C     STATUS   - I       SUCCESS_P OR FAILURE_P IF READ/WRITE PROBLEM
C                        NOTE THAT NOT OPENING ONE OR BOTH OF THE
C                        INPUT FILES IS NOT AN ERROR.
C
C   INPUT FILES:
C     WIND FILE- WIND_DIR/FILENAME WHERE FILENAME IS CREATED FROM
C                ATIME & CRES.
C     TEMP FILE- TEMP_DIR/FILENAME WHERE FILENAME IS CREATED FROM
C                ATIME & CRES
C     QC FILE  - QC_DIR/FILENAME WHERE FILENAME IS CREATED FROM THE
C                HOURLY PORTION OF ATIME
C     QC_CDL   - CDL FOR A NEW QC FILE
C     MAPVARS  - FILE LISTING NON-SPECIAL-HANDLING VARIABLES
C
C   OUTPUT FILES:
C     QC FILE  - QC_DIR/FILENAME WHERE FILENAME IS CREATED FROM OTIME
C
C SUBPROGRAMS CALLED:
C     LIBRARY: - NETCDF LIBRARY:
C                  NCCLOS
C                  NCDINQ
C                  NCINQ
C                  NCOPN
C              - SYSTEM LIBRARY:
C                  SYSTEM
C              - MADIS LIBRARY:
C                  M_GETLUN,M_LSTCHR,M_MKWFNM
C     UNIQUE:  MAPTREC,MAPLVLS,MAPVPROC,MWTNUM,MAPOBTIME,
C              MAPWINDS,MAPTEMPS
C
C REMARKS:
C
C   NONE (THIS IS *TOO* COMPLICATED FOR SIMPLE REMARKS...)
C
C ATTRIBUTES:
C   LANGUAGE: FORTRAN-77 + EXTENSIONS
C   MACHINE:  UNIX
C$$$


      SUBROUTINE MKMAPFILE (ATIME,CRES,OTIME,LSTRCQ,NRECW,NRECT,NRECI,
     1                      STATUS)


C     ------------------------------------------------------------------
C     INCLUDES AND COMMONS
C     ------------------------------------------------------------------
      INCLUDE 'IMPLICIT'
      INCLUDE 'INTQC1CON2'
      INCLUDE 'MAPCON'
      INCLUDE 'NETCDFI'
      INCLUDE 'STDOUT'
      INCLUDE 'MAPCOM'

C     ------------------------------------------------------------------
C     ARGUMENTS
C     ------------------------------------------------------------------
      CHARACTER*9   ATIME,OTIME
      CHARACTER*4   CRES
      INTEGER       LSTRCQ,NRECW,NRECT,NRECI,STATUS


C     ------------------------------------------------------------------
C     LOCAL
C     ------------------------------------------------------------------
      INTEGER I,J,K,L,M_LSTCHR,N,LUN,IOSTAT
      CHARACTER*14 FNAME
      CHARACTER*(MAXNCNAM) DIMNAM
      CHARACTER*73 LINE
      INTEGER NDIMS,NGATTS,NVARS,RCODE
      REAL AVGMIN(MXMAP_P)

      
C     *****************************************************************
C     ******************* BEGIN OF MKMAPFILE **************************
C     *****************************************************************

      STATUS = FAILURE_P

C     ------------------------------------------------------------------
C     OPEN THE POSSIBLE PAIR OF INPUT FILES AND GET THE NUMBER OF
C     INPUT RECORDS IN THEM.
C     ------------------------------------------------------------------
      FNAME = ATIME//CRES//'o'
      NRECW = 0
      NRECT = 0
      NRECI = 0
      WRITE(*,*)'OPENING: ','WIND_DIR/'//FNAME//'_wind.nc'
      NCIDW = NCOPN('WIND_DIR/'//FNAME//'_wind.nc', NCNOWRIT, RCODE)
      IF(RCODE.EQ.SUCCESS_P)THEN
         CALL NCINQ(NCIDW,NDIMS,NVARS,NGATTS,RECDIMW,RCODE)
         IF(RCODE.NE.0)THEN
            WRITE(STDOUT,*)'          - W NCINQ ERROR: ',RCODE
            GO TO 900
         ENDIF
         CALL NCDINQ(NCIDW,RECDIMW,DIMNAM,NRECW,RCODE)
         IF(RCODE.NE.0)THEN
            WRITE(STDOUT,*)'          - W NCDINQ RECDIM ERROR: ',RCODE
            GO TO 900
         ENDIF
      ENDIF

      WRITE(*,*)'OPENING: ','TEMP_DIR/'//FNAME//'_rass.nc'
      NCIDT = NCOPN('TEMP_DIR/'//FNAME//'_rass.nc', NCNOWRIT, RCODE)
      IF(RCODE.EQ.SUCCESS_P)THEN
         CALL NCINQ(NCIDT,NDIMS,NVARS,NGATTS,RECDIMT,RCODE)
         IF(RCODE.NE.0)THEN
            WRITE(STDOUT,*)'          - T NCINQ ERROR: ',RCODE
            GO TO 900
         ENDIF
         CALL NCDINQ(NCIDT,RECDIMT,DIMNAM,NRECT,RCODE)
         IF(RCODE.NE.0)THEN
            WRITE(STDOUT,*)'          - T NCDINQ RECDIM ERROR: ',RCODE
            GO TO 900
         ENDIF
      ENDIF
      
C     ------------------------------------------------------------------
C     IF THERE'S NOTHING TO PROCESS, RETURN WITH SUCCESS.
C     ------------------------------------------------------------------
      IF(NRECW.EQ.0.AND.NRECT.EQ.0)GO TO 850

C     ------------------------------------------------------------------
C     IF THIS IS THE FIRST TIME WE'RE HERE (WITH ANY DATA) CREATE AND
C     OPEN THE OUTPUT QC NETCDF FILE.  IF WE'VE ALREADY BEEN HERE, JUST
C     OPEN IT.
C     ------------------------------------------------------------------
      CALL M_MKWFNM(OTIME, FNAME, RCODE)
      IF(RCODE .NE. SUCCESS_P)GO TO 900
      FNAME(14:14) = ' '

      IF(LSTRCQ.NE.0)THEN

         NCIDQ = NCOPN('QC_DIR/'//FNAME, NCWRITE, RCODE)

      ELSE

         WRITE(STDOUT,*)'     - CREATING QC_DIR/'//FNAME
         CALL SYSTEM('ncgen -o QC_DIR/'//FNAME//' QC_CDL')
         NCIDQ = NCOPN('QC_DIR/'//FNAME, NCWRITE, RCODE)
         IF(RCODE .NE. SUCCESS_P)THEN
            WRITE(STDOUT,*)'          - NCOPN ERROR: ',RCODE
            GO TO 900
         ENDIF

      ENDIF

C     ------------------------------------------------------------------
C     GET THE RECORD DIMENSION.
C     ------------------------------------------------------------------
      CALL NCINQ(NCIDQ,NDIMS,NVARS,NGATTS,RECDIMQ,RCODE)
      IF(RCODE.NE.0)THEN
         WRITE(STDOUT,*)'          - Q NCINQ ERROR: ',RCODE
         GO TO 900
      ENDIF

C     ------------------------------------------------------------------
C     GET THE NUMBER OF RECORDS IN THE INTEGRATED SET OF INPUT RECORDS,
C     AND THE INDICES OF THE TEMPERATURE RECORDS IN THE INTEGRATED SET.
C     HANDLE THE OUTPUT VARIABLES READ ALONG THE WAY.
C     ------------------------------------------------------------------
      CALL MAPTREC(NRECW,NRECT,LSTRCQ,NRECI,RCODE)
      IF(RCODE.NE.SUCCESS_P)GO TO 900
      
C     ------------------------------------------------------------------
C     GET THE INDICES OF THE LEVELS IN THE INTEGRATED PROFILES, AND
C     HANDLE THE OUTPUT OF VARIABLES READ ALONG THE WAY.
C     ------------------------------------------------------------------
      CALL MAPLVLS(NRECW,NRECT,LSTRCQ,NRECI,RCODE)
      IF(RCODE.NE.SUCCESS_P)GO TO 900

C     ------------------------------------------------------------------
C     PROCESS THE VARIABLES THAT DON'T REQUIRE ANY SPECIAL HANDLING.
C     THESE ARE READ IN FROM THE MAPVARS FILE.  SEE THAT FILE, OR
C     THE MAPVPROC SUBROUTINE FOR DETAILS ON WHAT'S IN MAPVARS.
C     ------------------------------------------------------------------
      CALL M_GETLUN(LUN,1)
      OPEN(UNIT=LUN,FILE='MAPVARS',STATUS='OLD',IOSTAT=IOSTAT)
      IF(IOSTAT.NE.0)THEN
         WRITE(STDOUT,*)'          - FAILED TO OPEN MAPVARS'
         GO TO 900
      ENDIF

      DO I = 1, 1000

         READ(LUN,1,IOSTAT=IOSTAT,END=500)LINE
 1       FORMAT(A)
         IF(IOSTAT.NE.0)THEN
            WRITE(STDOUT,*)'          - FAILED TO READ MAPVARS'
            GO TO 900
         ENDIF
         IF(LINE(1:1).EQ.'B'.OR.LINE(1:1).EQ.'T'.OR.
     1      LINE(1:1).EQ.'W')THEN

            J = M_LSTCHR(LINE(6:35),30)
            K = M_LSTCHR(LINE(37:66),30)
C     ------------------------------------------------------------------
C     DOUBLE CASE IS HANDLED LIKE TIME
C     ALLOWED IS RECNUM AND MODE ONLY
C     ------------------------------------------------------------------
            IF(LINE(3:3).EQ.'6'.AND.LINE(1:1).EQ.'W'
     1           .AND.LINE(71:71).EQ.' '.AND.LINE(73:73).EQ.' ')THEN
               CALL MAPOBTIME(NRECW,0,LSTRCQ,NRECI,LINE(6:6+J-1),
     1              LINE(37:37+K-1),RCODE)
               IF(RCODE.NE.SUCCESS_P)GO TO 900
            ELSE IF(LINE(3:3).EQ.'6'.AND.LINE(1:1).EQ.'T'
     1           .AND.LINE(71:71).EQ.' '.AND.LINE(73:73).EQ.' ')THEN
               CALL MAPOBTIME(0,NRECT,LSTRCQ,NRECI,LINE(6:6+J-1),
     1              LINE(37:37+K-1),RCODE)
               IF(RCODE.NE.SUCCESS_P)GO TO 900
            ELSE
               CALL MAPVPROC(NRECW,NRECT,LSTRCQ,NRECI,LINE(1:1),
     1              LINE(3:3),LINE(6:6+J-1),LINE(37:37+K-1),LINE(71:71),
     2              LINE(73:73),RCODE)
               IF(RCODE.NE.SUCCESS_P)GO TO 900
            ENDIF
         ENDIF

      ENDDO

 500  CLOSE(UNIT=LUN)
      CALL M_GETLUN(LUN,2)

C     ------------------------------------------------------------------
C     FILL IN THE AVERAGING TIME PERIOD VARIABLE.  NOTE THAT THE
C     5-MINUTE RESOLUTION DATA ARE ACTUALLY 15-MINUTE AVERAGES (CREATED 
C     EVERY 5 MINUTES).
C     ------------------------------------------------------------------
      READ(CRES,2)K
 2    FORMAT(I4)
      IF(K.EQ.100)THEN
         K = 60
      ELSE IF(K.EQ.5)THEN
         K = 15
      ENDIF

      DO I = 1, NRECI

         AVGMIN(I) = K

      ENDDO

      CALL MWTNUM('averageMinutes',NCIDQ,RECDIMQ,LSTRCQ+1,NRECI,MISS_P,
     1            AVGMIN,RCODE,MXMAP,
     2 RMAPDATA,IMAPDATA,I2MAPDATA,BMAPDATA)
      IF(RCODE.NE.0)GO TO 900
         
C     ------------------------------------------------------------------
C     HANDLE THE OBSERVATION TIMES.  
C     ------------------------------------------------------------------
      CALL MAPOBTIME(NRECW,NRECT,LSTRCQ,NRECI,'timeObs',
     1               'observationTime',RCODE)
      IF(RCODE.NE.SUCCESS_P)GO TO 900

C     ------------------------------------------------------------------
C     HANDLE THE WINDS AND THEIR QC.
C     ------------------------------------------------------------------
      CALL MAPWINDS(NRECW,NRECI,LSTRCQ,RCODE)
      IF(RCODE.NE.SUCCESS_P)GO TO 900

C     ------------------------------------------------------------------
C     HANDLE THE TEMPERATURES AND THEIR QC.
C     ------------------------------------------------------------------
      CALL MAPTEMPS(NRECT,NRECI,LSTRCQ,RCODE)
      IF(RCODE.NE.SUCCESS_P)GO TO 900

C     ------------------------------------------------------------------
C     NOW THAT ALL'S SAID AND DONE, UPDATE LSTRCQ.
C     ------------------------------------------------------------------
      LSTRCQ = LSTRCQ + NRECI

 850  STATUS = SUCCESS_P

 900  CALL NCCLOS(NCIDW,RCODE)
      CALL NCCLOS(NCIDT,RCODE)
      CALL NCCLOS(NCIDQ,RCODE)

      RETURN
      END
