C$$$  SUBPROGRAM DOCUMENTATION BLOCK
C                .      .    .                                       .
C SUBPROGRAM:    MKSMRYMAP   SUMMARIZE MAP QC RESULTS
C   PRGMMR: M. BARTH         ORG: FSL        DATE: 02-04-22
C
C ABSTRACT: SUMMARIZE MAP QC RESULTS INTO DESCRIPTORS
C
C PROGRAM HISTORY LOG:
C   02-04-22  M. BARTH    ORIGINAL VERSION 
C   08-06-20  L. BENJAMIN CHANGED MADISCON TO INTQC1CON2 AND 
C                         MAPCON.
C   08-08-19  L. BENJAMIN CHANGED FMC_P TO MC_P
C
C USAGE:    CALL MKSMRYMAP (TYPE,NDLEV,NDSTA,NRECS,STAID,QCAPP,QCRES,
C                           QCDESC,RSAQC)
C
C   INPUT ARGUMENT LIST:
C     TYPE     - I       1 - WIND, 2 - RASS TEMPERATURES
C     NDLEV    - I       SIZE TO USE FOR LEVEL DIMENSION IN ARRAYS
C     NDSTA    - I       SIZE TO USE FOR STATION DIMENSION IN ARRAYS
C     NRECS    - I       NUMBER OF OB RECORDS
C     STAID    - CA      STATION NAMES FOR THE OB RECORDS
C     QCAPP    - IA      QC APPLIED WORDS FOR THE OB RECORDS
C     QCRES    - IA      QC RESULTS WORDS FOR THE OB RECORDS
C
C   INPUT/OUTPUT ARGUMENT LIST:
C     QCDESC   - CA      QC DESCRIPTORS SUMMARIZING THE RESULTS
C
C   OUTPUT ARGUMENT LIST:
C     RSAQC    - IA      RSA-STYLE QC FLAGS (0=GOOD,1=BAD)
C
C SUBPROGRAMS CALLED:
C     MADISLIB: - M_CHCKQC
C     UNIQUE:  - RDARFLMAP
C
C REMARKS:
C     NONE
C
C ATTRIBUTES:
C   LANGUAGE: FORTRAN-77 + EXTENSIONS
C   MACHINE:  UNIX
C$$$






      SUBROUTINE MKSMRYMAP (TYPE,NDLEV,NDSTA,NRECS,STAID,QCAPP,QCRES,
     1                      QCDESC,RSAQC)





C     ------------------------------------------------------------------
C     INCLUDES AND COMMONS
C     ------------------------------------------------------------------
      INCLUDE 'IMPLICIT'
      INCLUDE 'INTQC1CON2'
      INCLUDE 'MAPCON'
      INCLUDE 'STDOUT'


C     ------------------------------------------------------------------
C     ARGUMENTS
C     ------------------------------------------------------------------
      INTEGER       TYPE
      INTEGER       NDLEV
      INTEGER       NDSTA
      INTEGER       NRECS
      CHARACTER*6   STAID(NDSTA)
      INTEGER       QCAPP(NDLEV,NDSTA)
      INTEGER       QCRES(NDLEV,NDSTA)
      CHARACTER*1   QCDESC(NDLEV,NDSTA)
      INTEGER       RSAQC(NDLEV,NDSTA)

C     ------------------------------------------------------------------
C     LOCAL
C     ------------------------------------------------------------------
      INTEGER I,J,K,L,STATUS,NSTAGES
      LOGICAL FAIL(3),APP(3)
      PARAMETER (NSTAGES = 3)

      INTEGER NREJECT
      INTEGER OB_REJ(MXMAPLEV_P,MXMAP_P,2)
      CHARACTER*5 ID_REJ(MXMAP_P)

      INTEGER NACCEPT
      INTEGER OB_ACC(MXMAPLEV_P,MXMAP_P,2)
      CHARACTER*5 ID_ACC(MXMAP_P)


C     *****************************************************************
C     ******************* BEGIN OF MKSMRYMAP **************************
C     *****************************************************************


C     ------------------------------------------------------------------
C     GET THE ACCEPT AND REJECT LISTS.
C     ------------------------------------------------------------------
      NACCEPT = 0
      WRITE (STDOUT, 4900)'ACCEPT'
 4900 FORMAT(/7X,'     - CALLING RDARFLMAP TO READ ',A6,' LIST'/)
      CALL RDARFLMAP(1,STDOUT,MXMAP_P,MXMAPLEV_P,NACCEPT,ID_ACC,
     1               OB_ACC,STATUS)
      NREJECT = 0
      WRITE (STDOUT, 4900)'REJECT'
      CALL RDARFLMAP(2,STDOUT,MXMAP_P,MXMAPLEV_P,NREJECT,ID_REJ,
     1               OB_REJ,STATUS)


C     ------------------------------------------------------------------
C     CREATE SINGLE-CHARACTER QC DESCRIPTORS.
C     NOTE THAT UNLIKE OTHER MADIS DATASETS, THE MAP DESCRIPTORS MAY
C     ALREADY HAVE A "B" IN THEM -- BECAUSE FSL/DD'S QC INCLUDES A
C     SUBJECTIVE QC-BAD INDICATION.  THEREFORE, THE DESCRIPTORS MUST
C     ALREADY HAVE BEEN INITIALIZED BEFORE BEING SENT INTO THIS
C     ROUTINE.  ALL WE DO IS SCOOP UP THE FINAL RESULTS.
C     ------------------------------------------------------------------
      WRITE(STDOUT,*)'     - CREATING DESCRIPTORS'

      DO 10 J = 1, NRECS

         DO 20 I = 1, NDLEV

C     ------------------------------------------------------------------
C     APPLY SUBJECTIVE QC IF SPECIFIED IN THE ACCEPT/REJECT LISTS:
C
C     - G = SUBJECTIVE GOOD
C     - B = SUBJECTIVE BAD
C
C     NOTES:
C
C     - THE REJECT LIST TAKES PRECEDENCE IN CASE OF OPERATOR STUPIDITY.
C     - IF THE CAPABILITY TO PERFORM SUBJECTIVE QC IS ADDED *OUTSIDE* OF
C       THIS SUBROUTINE (E.G., VIA A WORKSTATION APPLICATION), LOGIC
C       CHANGES MAY BE REQUIRED SO THAT THE ACCEPT/REJECT LIST SETTINGS
C       DON'T OVERRIDE THE OUTSIDE APPLICATION SETTINGS, OR VICE VERSA.
C     ------------------------------------------------------------------
            DO 21 K = 1, NACCEPT
               IF(STAID(J)(1:5).EQ.ID_ACC(K))THEN

                  IF(OB_ACC(I,K,TYPE).EQ.1.AND.QCAPP(I,J).NE.0.AND.
     1               QCDESC(I,J).NE.'B')THEN
                     WRITE(STDOUT,4092)I,STAID(J)(1:5)
 4092                FORMAT(7X,' - ACCEPTING LEVEL ',I2,' FOR ',A)
                     QCDESC(I,J) = 'G'
                     RSAQC(I,J) = 0
                  ENDIF
                  GO TO 22
               ENDIF
 21         CONTINUE

 22         DO 23 K = 1, NREJECT
               IF(STAID(J)(1:5).EQ.ID_REJ(K))THEN

                  IF(OB_REJ(I,K,TYPE).EQ.1.AND.QCAPP(I,J).NE.0)THEN
                     WRITE(STDOUT,4093)I,STAID(J)(1:5)
 4093                FORMAT(7X,' - REJECTING LEVEL ',I2,' FOR ',A)
                     QCDESC(I,J) = 'B'
                     RSAQC(I,J) = 1
                  ENDIF
                  GO TO 24
               ENDIF
 23         CONTINUE

 24         IF(QCDESC(I,J).EQ.'G'.OR.QCDESC(I,J).EQ.'B')GO TO 20


C     ------------------------------------------------------------------
C     DETERMINE WHICH QC STAGES WERE APPLIED AND FAILED.
C     ------------------------------------------------------------------
            DO 25 K = 1, NSTAGES
               FAIL(K) = .FALSE.
               APP(K) = .FALSE.
 25         CONTINUE

C     ------------------------------------------------------------------
C     STAGE 1:  VALIDITY
C     ------------------------------------------------------------------
            CALL M_CHCKQC(QCAPP(I,J),VC_P,K,STATUS)
            IF(K.NE.0)APP(1) = .TRUE.
            CALL M_CHCKQC(QCRES(I,J),VC_P,K,STATUS)
            IF(K.NE.0)FAIL(1) = .TRUE.

C     ------------------------------------------------------------------
C     STAGE 2:  INTERNAL CONSISTENCY, TIME-HEIGHT CONSISTENCY,
C               MODEL CONSISTENCY
C     ------------------------------------------------------------------
            CALL M_CHCKQC(QCAPP(I,J),IC_P,K,STATUS)
            IF(K.NE.0)APP(2) = .TRUE.
            CALL M_CHCKQC(QCRES(I,J),IC_P,K,STATUS)
            IF(K.NE.0)FAIL(2) = .TRUE.

            CALL M_CHCKQC(QCAPP(I,J),THC_P,K,STATUS)
            IF(K.NE.0)APP(2) = .TRUE.
            CALL M_CHCKQC(QCRES(I,J),THC_P,K,STATUS)
            IF(K.NE.0)FAIL(2) = .TRUE.

            CALL M_CHCKQC(QCAPP(I,J),MC_P,K,STATUS)
            IF(K.NE.0)APP(2) = .TRUE.
            CALL M_CHCKQC(QCRES(I,J),MC_P,K,STATUS)
            IF(K.NE.0)FAIL(2) = .TRUE.

C     ------------------------------------------------------------------
C     STAGE 3:  SPATIAL CONSISTENCY
C     ------------------------------------------------------------------
            CALL M_CHCKQC(QCAPP(I,J),SC_P,K,STATUS)
            IF(K.NE.0)APP(3) = .TRUE.
            CALL M_CHCKQC(QCRES(I,J),SC_P,K,STATUS)
            IF(K.NE.0)FAIL(3) = .TRUE.

C     ------------------------------------------------------------------
C     NOW HANDLE THIS CASE:  WITH THE DESCRIPTORS AS CURRENTLY DEFINED
C     IF STAGE 3 HAS BEEN APPLIED, WE HAVE TO ASSUME THAT STAGES
C     1 AND 2 HAVE BEEN APPLIED AS WELL.  WITH STAGE 1 THIS IS TRUE, BUT
C     DEPENDING ON THE DATA SOURCE, STAGE 2 HASN'T NECESSARILY BEEN 
C     APPLIED, AS IT DEPENDS ON VARIOUS THINGS.  THUS, IF STAGE 3 HAS 
C     BEEN APPLIED AND STAGE 2 HASN'T, SET APP(2) TO PRETEND THAT IT 
C     HAS BEEN.
C     ------------------------------------------------------------------
            IF(APP(3))APP(2) = .TRUE.


C     ------------------------------------------------------------------
C     CONSOLIDATE THE RESULTS OF THE AUTOMATED QC:
C     - V = PASSED 1,2,3
C     - Q = PASSED 1, FAILED 2 OR 3
C     - S = PASSED 1, 2
C     - C = PASSED 1
C     - X = FAILED 1
C     - Z = NO QC
C
C     * NOTE THAT THE SUBJECTIVE TESTS SHOULD BE DONE FIRST, AS THEY
C     OVERRIDE THE OTHERS.  OTHERWISE, NO ATTEMPT HAS BEEN MADE TO
C     LOGICALLY TRAVERSE DOWN THIS TREE.  THUS, THE CODE CAN BE
C     MODIFIED WITHOUT WORRYING ABOUT PRECEDENCE OF THE TESTS.
C     ------------------------------------------------------------------
            IF(APP(1).AND.APP(2).AND.APP(3).AND..NOT.FAIL(1).AND.
     1         .NOT.FAIL(2).AND..NOT.FAIL(3))THEN

               QCDESC(I,J) = 'V'
               RSAQC(I,J) = 0

            ELSE IF(APP(1).AND..NOT.FAIL(1).AND.((APP(2).AND.FAIL(2))
     1              .OR.(APP(3).AND.FAIL(3))))THEN

               QCDESC(I,J) = 'Q'
               RSAQC(I,J) = 1

            ELSE IF(APP(1).AND.APP(2).AND..NOT.APP(3)
     1              .AND..NOT.FAIL(1).AND..NOT.FAIL(2))THEN

               QCDESC(I,J) = 'S'
               RSAQC(I,J) = 0

            ELSE IF(APP(1).AND..NOT.APP(2).AND..NOT.APP(3).AND.
     1              .NOT.FAIL(1))THEN

               QCDESC(I,J) = 'C'
               RSAQC(I,J) = 0

            ELSE IF(APP(1).AND.FAIL(1))THEN

               QCDESC(I,J) = 'X'
               RSAQC(I,J) = 1

            ELSE IF(.NOT.APP(1).AND..NOT.APP(2).AND..NOT.APP(3))THEN

               QCDESC(I,J) = 'Z'
               RSAQC(I,J) = MISS_P

            ELSE

               WRITE(STDOUT,123)I,J,QCRES(I,J),QCAPP(I,J)
 123           FORMAT('        - LOGIC ERROR AT LEVEL ',I2,' RECORD ',
     1                I5,' QCR ',I5,' QCA ',I5)
            ENDIF

 20      CONTINUE

 10   CONTINUE

      RETURN
      END
