C$$$  SUBPROGRAM DOCUMENTATION BLOCK
C                .      .    .                                       .
C SUBPROGRAM:    RDMAPOBS    READ MAP OBS, QC INFO, AND ID INFO
C   PRGMMR: M. BARTH         ORG: FSL        DATE: 02-04-22
C
C ABSTRACT: INPUT DATA TO BE QC'ED FROM A SINGLE MAP NETCDF FILE.
C
C PROGRAM HISTORY LOG:
C   02-04-22  M. BARTH    ORIGINAL VERSION
C   02-06-26  M. BARTH    ADDED DTIME
C   08-06-20  L. BENJAMIN CHANGED MADISCON TO INTQC1CON2.  CHANGE 
C                         MRDNUM CALL TO LOCAL RDNUM.
C
C USAGE:    CALL RDMAPOBS (ATIME,QCDIR,NDLEV,NDSTA,NDVAR,NRECS,U,V,T,
C                          HT,LT,STAID,DTIME,QCAPP,QCRES,ICAPP,ICRES,
C                          QCDESC,RSAQC,STATUS)
C
C   INPUT ARGUMENT LIST:
C     ATIME    - C       YYJJJHHMM OF HOUR TO BE PROCESSED
C     QCDIR    - C       NAME OF THE DIRECTORY HOLDING QC FILES
C     NDLEV    - I       SIZE TO USE FOR LEVEL DIMENSION IN ARRAYS
C     NDSTA    - I       SIZE TO USE FOR STATION DIMENSION IN ARRAYS
C     NDVAR    - I       SIZE TO USE FOR VARIABLE DIMENSION IN ARRAYS
C
C   INPUT/OUTPUT ARGUMENT LIST:
C
C     NRECS    - I       NUMBER OF OB RECORDS TO READ
C
C   OUTPUT ARGUMENT LIST:
C     U        - RA      U WIND COMPONENTS
C     V        - RA      V WIND COMPONENTS
C     T        - RA      VIRTUAL TEMPERATURES
C     HT       - RA      HEIGHTS - ABOVE GROUND (NOT ABOVE STATION)
C     LT       - RA      LEVEL TYPES
C     STAID    - CA      STATION NAMES
C     DTIME    - DA      STORED OB TIMES (NOT I4TIMES)
C     QCAPP    - IA      QC APPLIED WORDS
C     QCRES    - IA      QC RESULTS WORDS
C     ICAPP    - IA      IC (INTERNAL CONSISTENCY CHECK) APPLIED
C                        WORDS
C     ICRES    - IA      IC RESULTS WORDS
C     QCDESC   - CA      QC DATA DESCRIPTORS
C     RSAQC    - IA      RSA-STYLE QC FLAGS
C     STATUS   - I       SUCCESS_P OR FAILURE_P IF INPUT FILE PROBLEM
C
C   INPUT FILES:
C     QC FILE  - QCDIR/FILENAME WHERE FILENAME IS CREATED FROM ATIME
C
C SUBPROGRAMS CALLED:
C     LIBRARY: - NETCDF LIBRARY:
C                NCCLOS
C                NCOPN
C              - MADISLIB:
C                MRDCHR                
C                MREAL2INT
C                M_LSTCHR
C                M_MKWFNM
C     UNIQUE: RDNUM, RDDBL
C
C REMARKS:
C   THE DIMENSIONS OF THE LOCAL ARRAY RDATA MUST MATCH NDLEV,NDSTA.
C
C ATTRIBUTES:
C   LANGUAGE: FORTRAN-77 + EXTENSIONS
C   MACHINE:  UNIX
C$$$


      SUBROUTINE RDMAPOBS (ATIME,QCDIR,NDLEV,NDSTA,NDVAR,NRECS,U,V,T,HT,
     1                     LT,STAID,DTIME,QCAPP,QCRES,ICAPP,ICRES,
     2                     QCDESC,RSAQC,STATUS)





C     ------------------------------------------------------------------
C     INCLUDES AND COMMONS
C     ------------------------------------------------------------------
      INCLUDE 'IMPLICIT'
      INCLUDE 'INTQC1CON2'
      INCLUDE 'MAPCON'
      INCLUDE 'NETCDFI'
      INCLUDE 'STDOUT'


C     ------------------------------------------------------------------
C     ARGUMENTS
C     ------------------------------------------------------------------
      CHARACTER*9   ATIME
      CHARACTER*(*) QCDIR
      INTEGER       NDLEV
      INTEGER       NDSTA
      INTEGER       NDVAR
      INTEGER       NRECS
      REAL          U(NDLEV,NDSTA)
      REAL          V(NDLEV,NDSTA)
      REAL          T(NDLEV,NDSTA)
      REAL          HT(NDLEV,NDSTA)
      REAL          LT(NDLEV,NDSTA)
      CHARACTER*6   STAID(NDSTA)
      DOUBLE PRECISION DTIME(NDSTA)
      INTEGER       QCAPP(NDLEV,NDSTA,NDVAR)
      INTEGER       QCRES(NDLEV,NDSTA,NDVAR)
      INTEGER       ICAPP(NDLEV,NDSTA,NDVAR)
      INTEGER       ICRES(NDLEV,NDSTA,NDVAR)
      CHARACTER*1   QCDESC(NDLEV,NDSTA,NDVAR)
      INTEGER       RSAQC(NDLEV,NDSTA,NDVAR)
      INTEGER       STATUS


C     ------------------------------------------------------------------
C     LOCAL
C     ------------------------------------------------------------------
      INTEGER       I,J,M_LSTCHR
      CHARACTER*13  FNAME
      CHARACTER*60  DIRNAM
      INTEGER       NCID,RCODE,NDIMS,NVARS,NGATTS,RECDIM,NVATTS
      CHARACTER*(MAXNCNAM) DIMNAM
      REAL          RDATA(MXMAPLEV_P,MXMAP_P)
      REAL          ELEV(MXMAP_P)
      DOUBLE PRECISION DMISS
      INTEGER RDCNT

C     -----------------------------------------------------------------
C     DATA STATEMENTS (KEEP AT BOTTOM OF DECLARATIONS)
C     -----------------------------------------------------------------
      DATA DMISS/0.0D0/


C     *****************************************************************
C     ******************* BEGIN OF RDMAPOBS ***************************
C     *****************************************************************

      STATUS = FAILURE_P

C     ------------------------------------------------------------------
C     CREATE THE FILENAME FROM ATIME
C     ------------------------------------------------------------------
      WRITE(STDOUT,*)'     - CREATING QC FILENAME FROM ATIME: ',ATIME
      CALL M_MKWFNM(ATIME, FNAME, RCODE)
      IF(RCODE .NE. SUCCESS_P)RETURN

C     ------------------------------------------------------------------
C     OPEN THE QC FILE FOR THIS TIME.
C     ------------------------------------------------------------------
      DIRNAM = QCDIR
      I = M_LSTCHR(DIRNAM,60)
      IF(I.EQ.0)I = 60
      WRITE(STDOUT,*)'     - OPENING '//DIRNAM(1:I)//'/'//FNAME
      NCID = NCOPN(DIRNAM(1:I)//'/'//FNAME, NCWRITE, RCODE)
      IF(RCODE .NE. SUCCESS_P)THEN
         WRITE(STDOUT,*)'          - ERROR OPENING FILE: ',RCODE
         RETURN
      ENDIF

C     ------------------------------------------------------------------
C     READ THE INFO THE CALLER WANTS, AND TRANSFER FROM RDNUM'S REAL
C     ARRAYS WHEN THE CALLER'S DATA TYPE DIFFERS.  NOTE THAT WHEN WE
C     READ THE WIND QC STRUCTURES, WE ONLY GET THOSE FOR THE U COMP:
C     BOTH COMPONENTS HAVE QC STRUCTURES SO THAT ALL CODE WILL WORK,
C     BUT THEY BOTH HAVE THE SAME INFO AND ONLY ONE SET IS USED IN
C     THIS PROGRAM.
C     ------------------------------------------------------------------
      CALL RDNUM('uComponent',NCID,RECDIM,1,NRECS,MISS_P,U,RDCNT,
     1     STATUS)
      IF(STATUS .NE. SUCCESS_P)GO TO 900

      CALL RDNUM('vComponent',NCID,RECDIM,1,NRECS,MISS_P,V,RDCNT,
     1     STATUS)
      IF(STATUS .NE. SUCCESS_P)GO TO 900

      CALL RDNUM('temperature',NCID,RECDIM,1,NRECS,MISS_P,T,RDCNT,
     1     STATUS)
      IF(STATUS .NE. SUCCESS_P)GO TO 900

      CALL RDNUM('levels',NCID,RECDIM,1,NRECS,MISS_P,HT,RDCNT,STATUS)
      IF(STATUS .NE. SUCCESS_P)GO TO 900

      CALL RDNUM('windRASSMode',NCID,RECDIM,1,NRECS,MISS_P,LT,RDCNT,
     1     STATUS)
      IF(STATUS .NE. SUCCESS_P)GO TO 900

      CALL RDNUM('uComponentQCA',NCID,RECDIM,1,NRECS,MISS_P,RDATA,
     1     RDCNT,STATUS)
      IF(STATUS .NE. SUCCESS_P)GO TO 900
      CALL MREAL2INT(NRECS*NDLEV,RDATA,QCAPP)

      CALL RDNUM('uComponentQCR',NCID,RECDIM,1,NRECS,MISS_P,RDATA,
     1     RDCNT,STATUS)
      IF(STATUS .NE. SUCCESS_P)GO TO 900
      CALL MREAL2INT(NRECS*NDLEV,RDATA,QCRES)

      CALL RDNUM('uComponentICA',NCID,RECDIM,1,NRECS,MISS_P,RDATA,
     1     RDCNT,STATUS)
      IF(STATUS .NE. SUCCESS_P)GO TO 900
      CALL MREAL2INT(NRECS*NDLEV,RDATA,ICAPP)

      CALL RDNUM('uComponentICR',NCID,RECDIM,1,NRECS,MISS_P,RDATA,
     1     RDCNT,STATUS)
      IF(STATUS .NE. SUCCESS_P)GO TO 900
      CALL MREAL2INT(NRECS*NDLEV,RDATA,ICRES)

      CALL RDNUM('uQcFlag',NCID,RECDIM,1,NRECS,MISS_P,RDATA,RDCNT,
     1     STATUS)
      IF(STATUS .NE. SUCCESS_P)GO TO 900
      CALL MREAL2INT(NRECS*NDLEV,RDATA,RSAQC)

      CALL MRDCHR('uComponentDD',NCID,RECDIM,1,NRECS,QCDESC,I,
     1            STATUS)
      IF(STATUS .NE. SUCCESS_P)GO TO 900

      CALL RDNUM('temperatureQCA',NCID,RECDIM,1,NRECS,MISS_P,RDATA,
     1     RDCNT,STATUS)
      IF(STATUS .NE. SUCCESS_P)GO TO 900
      CALL MREAL2INT(NRECS*NDLEV,RDATA,QCAPP(1,1,2))

      CALL RDNUM('temperatureQCR',NCID,RECDIM,1,NRECS,MISS_P,RDATA,
     1     RDCNT,STATUS)
      IF(STATUS .NE. SUCCESS_P)GO TO 900
      CALL MREAL2INT(NRECS*NDLEV,RDATA,QCRES(1,1,2))

      CALL RDNUM('temperatureICA',NCID,RECDIM,1,NRECS,MISS_P,RDATA,
     1     RDCNT,STATUS)
      IF(STATUS .NE. SUCCESS_P)GO TO 900
      CALL MREAL2INT(NRECS*NDLEV,RDATA,ICAPP(1,1,2))

      CALL RDNUM('temperatureICR',NCID,RECDIM,1,NRECS,MISS_P,RDATA,
     1     RDCNT,STATUS)
      IF(STATUS .NE. SUCCESS_P)GO TO 900
      CALL MREAL2INT(NRECS*NDLEV,RDATA,ICRES(1,1,2))

      CALL RDNUM('tempQcFlag',NCID,RECDIM,1,NRECS,MISS_P,RDATA,RDCNT,
     1     STATUS)
      IF(STATUS .NE. SUCCESS_P)GO TO 900
      CALL MREAL2INT(NRECS*NDLEV,RDATA,RSAQC(1,1,2))

      CALL MRDCHR('temperatureDD',NCID,RECDIM,1,NRECS,QCDESC(1,1,2),I,
     1            STATUS)
      IF(STATUS .NE. SUCCESS_P)GO TO 900

      CALL MRDCHR('providerId',NCID,RECDIM,1,NRECS,STAID,I,STATUS)
      IF(STATUS .NE. SUCCESS_P)GO TO 900

      CALL RDDBL('observationTime',NCID,RECDIM,1,NRECS,DMISS,DTIME,
     1            STATUS)
      IF(STATUS .NE. SUCCESS_P)GO TO 900

      CALL RDNUM('elevation',NCID,RECDIM,1,NRECS,MISS_P,ELEV,RDCNT,
     1     STATUS)
      IF(STATUS .NE. SUCCESS_P)GO TO 900

C     ------------------------------------------------------------------
C     REPLACE THE NULL AT THE END OF STAID WITH A BLANK, AND ADD THE
C     STATION ELEVATIONS TO THE HEIGHTS.
C     ------------------------------------------------------------------
      DO 100 I = 1, NRECS
         STAID(I)(6:6) = ' '
         IF(ELEV(I).NE.MISS_P)THEN
            DO 101 J = 1, MXMAPLEV_P
               IF(HT(J,I).NE.MISS_P)HT(J,I) = HT(J,I) + ELEV(I)
 101        CONTINUE
         ENDIF
 100  CONTINUE

C     ------------------------------------------------------------------
C     FINISH UP:  CLOSE NETCDF FILE, ZERO NRECS IF AN ERROR.
C     ------------------------------------------------------------------
 900  CALL NCCLOS(NCID,RCODE)
      IF(STATUS .NE. SUCCESS_P)NRECS = 0

      RETURN
      END
