C$$$  SUBPROGRAM DOCUMENTATION BLOCK
C                .      .    .                                       .
C SUBPROGRAM:    WTMAP       OUTPUT MAP DATASET QC INFO TO NETCDF FILE
C   PRGMMR: M. BARTH         ORG: FSL        DATE: 02-04-25
C
C ABSTRACT: OUTPUT MAP QC INFO AND AWIPS INVENTORY INFO TO A SINGLE 
C   QC'ED NETCDF FILE.
C
C PROGRAM HISTORY LOG:
C   02-04-25  M. BARTH    ORIGINAL VERSION 
C   08-06-20  L. BENJAMIN CHANGED MADISCON TO INTQC1CON2.  ADDED
C                         TEMPORARY SPACE MEMORY FOR DOWN STREAM
C                         CALLS USAGE.
C
C USAGE:    CALL WTMAP (ATIME,NDLEV,NDSTA,NDVAR,NRECS,QCAPP,QCRES,
C                       ICAPP,ICRES,QCDESC,RSAQC,STATUS)
C
C   INPUT ARGUMENT LIST:
C     ATIME    - C       YYJJJHHMM OF HOUR TO BE PROCESSED
C     NDLEV    - I       SIZE TO USE FOR LEVEL DIMENSION IN ARRAYS
C     NDSTA    - I       SIZE TO USE FOR STATION DIMENSION IN ARRAYS
C     NDVAR    - I       SIZE TO USE FOR VARIABLE DIMENSION IN ARRAYS
C     NRECS    - I       NUMBER OF OB RECORDS
C     QCAPP    - IA      QC APPLIED WORDS FOR THE OB RECORDS
C     QCRES    - IA      QC RESULTS WORDS FOR THE OB RECORDS
C     ICAPP    - IA      IC (INTERNAL CONSISTENCY CHECK) APPLIED
C                        WORDS FOR THE OB RECORDS
C     ICRES    - IA      IC RESULTS WORDS FOR THE OB RECORDS
C     QCDESC   - CA      QC DATA DESCRIPTORS FOR THE OB RECORDS
C     RSAQC    - IA      RSA-STYLE QC FLAGS FOR THE OB RECORDS
C
C   OUTPUT ARGUMENT LIST:
C
C     STATUS   - I       SUCCESS_P OR FAILURE_P IF OUTPUT FILE PROBLEM
C
C   OUTPUT FILES:
C     QC FILE  - QC_DIR/FILENAME WHERE FILENAME IS CREATED FROM ATIME
C
C SUBPROGRAMS CALLED:
C     LIBRARY: - NETCDF:
C                NCCLOS
C                NCINQ
C                NCOPN
C              - MADIS LIBRARY:
C                M_LSTCHR
C                M_MKWFNM
C     UNIQUE:  - CDFINV,MWTNUM,MWTCHR
C
C REMARKS:
C   A SINGLE SET OF WIND QC STRUCTURES IS USED, BUT THESE APPLY
C   TO U/V, SPD/DIR VERSIONS OF THE DATA, SO WE WRITE THEM OUT
C   FOR ALL 4 VARIABLES.
C
C ATTRIBUTES:
C   LANGUAGE: FORTRAN-77 + EXTENSIONS
C   MACHINE:  UNIX
C$$$


      SUBROUTINE WTMAP (ATIME,NDLEV,NDSTA,NDVAR,NRECS,QCAPP,QCRES,
     1                  ICAPP,ICRES,QCDESC,RSAQC,STATUS)

C     ------------------------------------------------------------------
C     INCLUDES AND COMMONS
C     ------------------------------------------------------------------
      INCLUDE 'IMPLICIT'
      INCLUDE 'INTQC1CON2'
      INCLUDE 'INTQC2CON1'
      INCLUDE 'MAPCON'
      INCLUDE 'MAPCOM'
      INCLUDE 'NETCDFI'
      INCLUDE 'STDOUT'

C     ------------------------------------------------------------------
C     ARGUMENTS
C     ------------------------------------------------------------------
      CHARACTER*9   ATIME
      INTEGER       NDLEV
      INTEGER       NDSTA
      INTEGER       NDVAR
      INTEGER       NRECS
      INTEGER       QCAPP(NDLEV*NDSTA,NDVAR)
      INTEGER       QCRES(NDLEV*NDSTA,NDVAR)
      INTEGER       ICAPP(NDLEV*NDSTA,NDVAR)
      INTEGER       ICRES(NDLEV*NDSTA,NDVAR)
      CHARACTER*1   QCDESC(NDLEV*NDSTA,NDVAR)
      INTEGER       RSAQC(NDLEV*NDSTA,NDVAR)
      INTEGER       STATUS

C     ------------------------------------------------------------------
C     LOCAL
C     ------------------------------------------------------------------
      INTEGER       I,NOUT,M_LSTCHR
      CHARACTER*13  FNAME
      INTEGER       NCID,RCODE,RECDIM,NDIMS,NVARS,NGATTS
      REAL          RDATA(MXMAPLEV_P*MXMAP_P)
      CHARACTER*6   INVID(MXMAP_P)
      DOUBLE PRECISION INVTIME(MXMAP_P),DMISS

C     CDFINV STORAGE
      INTEGER MXTMPCI
      PARAMETER (MXTMPCI = MXMAP_P)

      INTEGER MAX_IDLENCI,MAX_BINSCI,MAX_RECCI
      PARAMETER (MAX_IDLENCI = 50)
      PARAMETER (MAX_RECCI = 500)
      PARAMETER (MAX_BINSCI = 32)

      INTEGER LAST_RECCI(MXTMPCI),PREV_RECCI(MXTMPCI)
      INTEGER INVENTORYCI(MXTMPCI),OFLOWCI(MXTMPCI)
      INTEGER TIMBINCI(MXTMPCI),INDXCI(MXTMPCI)
      CHARACTER*(MAX_IDLENCI) INVIDCI(MXTMPCI)
      BYTE BINVIDCI(MAX_IDLENCI,MXTMPCI)
      EQUIVALENCE(BINVIDCI,INVIDCI)
      CHARACTER*1 STATIC_IDSCI(MAX_IDLENCI*MXTMPCI)
      BYTE BSTATICCI(MAX_IDLENCI,MXTMPCI)
      EQUIVALENCE(BSTATICCI,STATIC_IDSCI)
      INTEGER ITEMPCI(MAX_RECCI)
      INTEGER FIRST_IN_BINCI(MAX_BINSCI),LAST_IN_BINCI(MAX_BINSCI)
      CHARACTER*(MAX_IDLENCI) LASTCI


C     -----------------------------------------------------------------
C     DATA STATEMENTS (KEEP AT BOTTOM OF DECLARATIONS)
C     -----------------------------------------------------------------
      DATA DMISS/0.0D0/

C     *****************************************************************
C     ******************* BEGIN OF WTMAP ******************************
C     *****************************************************************

      STATUS = FAILURE_P
      NOUT = NRECS * NDLEV

C     ------------------------------------------------------------------
C     CREATE THE FILENAME FROM ATIME
C     ------------------------------------------------------------------
      WRITE(STDOUT,*)'     - CREATING QC FILENAME FROM ATIME: ',ATIME
      CALL M_MKWFNM(ATIME, FNAME, RCODE)
      IF(RCODE .NE. SUCCESS_P)RETURN

C     ------------------------------------------------------------------
C     OPEN THE QC FILE FOR THIS TIME.
C     ------------------------------------------------------------------
      WRITE(STDOUT,*)'     - OPENING QC_DIR/'//FNAME
      NCID = NCOPN('QC_DIR/'//FNAME, NCWRITE, RCODE)
      IF(RCODE .NE. SUCCESS_P)THEN
         WRITE(STDOUT,*)'          - ERROR OPENING FILE: ',RCODE
         RETURN
      ENDIF

C     ------------------------------------------------------------------
C     GET THE RECORD DIMENSION ID.
C     ------------------------------------------------------------------
      CALL NCINQ(NCID,NDIMS,NVARS,NGATTS,RECDIM,RCODE)
      IF(RCODE.NE.0)THEN
         WRITE(STDOUT,*)'          - NCINQ ERROR: ',RCODE
         GO TO 900
      ENDIF

C     ------------------------------------------------------------------
C     THE PROCEDURE IS TO MOVE ANY INTEGER OUTPUTS INTO A REAL ARRAY SO 
C     WE CAN USE MWTNUM, THEN OUTPUT.
C     ------------------------------------------------------------------

C     ------------------------------------------------------------------
C     WINDS
C     ------------------------------------------------------------------
      CALL MWTCHR('uComponentDD',NCID,RECDIM,1,NRECS,QCDESC,RCODE)
      IF(RCODE.NE.0)GO TO 900
      CALL MWTCHR('vComponentDD',NCID,RECDIM,1,NRECS,QCDESC,RCODE)
      IF(RCODE.NE.0)GO TO 900
      CALL MWTCHR('windSpeedDD',NCID,RECDIM,1,NRECS,QCDESC,RCODE)
      IF(RCODE.NE.0)GO TO 900
      CALL MWTCHR('windDirDD',NCID,RECDIM,1,NRECS,QCDESC,RCODE)
      IF(RCODE.NE.0)GO TO 900

      DO I = 1, NOUT
         RDATA(I) = QCAPP(I,1)
      ENDDO
      CALL MWTNUM('uComponentQCA',NCID,RECDIM,1,NRECS,MISS_P,RDATA,
     1            RCODE,MXMAP,
     2 RMAPDATA,IMAPDATA,I2MAPDATA,BMAPDATA)
      IF(RCODE.NE.SUCCESS_P)GO TO 900
      CALL MWTNUM('vComponentQCA',NCID,RECDIM,1,NRECS,MISS_P,RDATA,
     1            RCODE,MXMAP,
     2 RMAPDATA,IMAPDATA,I2MAPDATA,BMAPDATA)
      IF(RCODE.NE.SUCCESS_P)GO TO 900
      CALL MWTNUM('windSpeedQCA',NCID,RECDIM,1,NRECS,MISS_P,RDATA,
     1            RCODE,MXMAP,
     2 RMAPDATA,IMAPDATA,I2MAPDATA,BMAPDATA)
      IF(RCODE.NE.SUCCESS_P)GO TO 900
      CALL MWTNUM('windDirQCA',NCID,RECDIM,1,NRECS,MISS_P,RDATA,
     1            RCODE,MXMAP,
     2 RMAPDATA,IMAPDATA,I2MAPDATA,BMAPDATA)
      IF(RCODE.NE.SUCCESS_P)GO TO 900

      DO I = 1, NOUT
         RDATA(I) = QCRES(I,1)
      ENDDO
      CALL MWTNUM('uComponentQCR',NCID,RECDIM,1,NRECS,MISS_P,RDATA,
     1            RCODE,MXMAP,
     2 RMAPDATA,IMAPDATA,I2MAPDATA,BMAPDATA)
      IF(RCODE.NE.SUCCESS_P)GO TO 900
      CALL MWTNUM('vComponentQCR',NCID,RECDIM,1,NRECS,MISS_P,RDATA,
     1            RCODE,MXMAP,
     2 RMAPDATA,IMAPDATA,I2MAPDATA,BMAPDATA)
      IF(RCODE.NE.SUCCESS_P)GO TO 900
      CALL MWTNUM('windSpeedQCR',NCID,RECDIM,1,NRECS,MISS_P,RDATA,
     1            RCODE,MXMAP,
     2 RMAPDATA,IMAPDATA,I2MAPDATA,BMAPDATA)
      IF(RCODE.NE.SUCCESS_P)GO TO 900
      CALL MWTNUM('windDirQCR',NCID,RECDIM,1,NRECS,MISS_P,RDATA,
     1            RCODE,MXMAP,
     2 RMAPDATA,IMAPDATA,I2MAPDATA,BMAPDATA)
      IF(RCODE.NE.SUCCESS_P)GO TO 900

      DO I = 1, NOUT
         RDATA(I) = ICAPP(I,1)
      ENDDO
      CALL MWTNUM('uComponentICA',NCID,RECDIM,1,NRECS,MISS_P,RDATA,
     1            RCODE,MXMAP,
     2 RMAPDATA,IMAPDATA,I2MAPDATA,BMAPDATA)
      IF(RCODE.NE.SUCCESS_P)GO TO 900
      CALL MWTNUM('vComponentICA',NCID,RECDIM,1,NRECS,MISS_P,RDATA,
     1            RCODE,MXMAP,
     2 RMAPDATA,IMAPDATA,I2MAPDATA,BMAPDATA)
      IF(RCODE.NE.SUCCESS_P)GO TO 900
      CALL MWTNUM('windSpeedICA',NCID,RECDIM,1,NRECS,MISS_P,RDATA,
     1            RCODE,MXMAP,
     2 RMAPDATA,IMAPDATA,I2MAPDATA,BMAPDATA)
      IF(RCODE.NE.SUCCESS_P)GO TO 900
      CALL MWTNUM('windDirICA',NCID,RECDIM,1,NRECS,MISS_P,RDATA,
     1            RCODE,MXMAP,
     2 RMAPDATA,IMAPDATA,I2MAPDATA,BMAPDATA)
      IF(RCODE.NE.SUCCESS_P)GO TO 900

      DO I = 1, NOUT
         RDATA(I) = ICRES(I,1)
      ENDDO
      CALL MWTNUM('uComponentICR',NCID,RECDIM,1,NRECS,MISS_P,RDATA,
     1            RCODE,MXMAP,
     2 RMAPDATA,IMAPDATA,I2MAPDATA,BMAPDATA)
      IF(RCODE.NE.SUCCESS_P)GO TO 900
      CALL MWTNUM('vComponentICR',NCID,RECDIM,1,NRECS,MISS_P,RDATA,
     1            RCODE,MXMAP,
     2 RMAPDATA,IMAPDATA,I2MAPDATA,BMAPDATA)
      IF(RCODE.NE.SUCCESS_P)GO TO 900
      CALL MWTNUM('windSpeedICR',NCID,RECDIM,1,NRECS,MISS_P,RDATA,
     1            RCODE,MXMAP,
     2 RMAPDATA,IMAPDATA,I2MAPDATA,BMAPDATA)
      IF(RCODE.NE.SUCCESS_P)GO TO 900
      CALL MWTNUM('windDirICR',NCID,RECDIM,1,NRECS,MISS_P,RDATA,
     1            RCODE,MXMAP,
     2 RMAPDATA,IMAPDATA,I2MAPDATA,BMAPDATA)
      IF(RCODE.NE.SUCCESS_P)GO TO 900

      DO I = 1, NOUT
         RDATA(I) = RSAQC(I,1)
      ENDDO
      CALL MWTNUM('wsQcFlag',NCID,RECDIM,1,NRECS,MISS_P,RDATA,RCODE,
     1     MXMAP,
     2 RMAPDATA,IMAPDATA,I2MAPDATA,BMAPDATA)
      IF(RCODE.NE.SUCCESS_P)GO TO 900
      CALL MWTNUM('wdQcFlag',NCID,RECDIM,1,NRECS,MISS_P,RDATA,RCODE,
     1     MXMAP,
     2 RMAPDATA,IMAPDATA,I2MAPDATA,BMAPDATA)
      IF(RCODE.NE.SUCCESS_P)GO TO 900
      CALL MWTNUM('uQcFlag',NCID,RECDIM,1,NRECS,MISS_P,RDATA,RCODE,
     1     MXMAP,
     2 RMAPDATA,IMAPDATA,I2MAPDATA,BMAPDATA)
      IF(RCODE.NE.SUCCESS_P)GO TO 900
      CALL MWTNUM('vQcFlag',NCID,RECDIM,1,NRECS,MISS_P,RDATA,RCODE,
     1     MXMAP,
     2 RMAPDATA,IMAPDATA,I2MAPDATA,BMAPDATA)
      IF(RCODE.NE.SUCCESS_P)GO TO 900

C     ------------------------------------------------------------------
C     TEMPERATURES
C     ------------------------------------------------------------------
      CALL MWTCHR('temperatureDD',NCID,RECDIM,1,NRECS,QCDESC(1,2),RCODE)
      IF(RCODE.NE.0)GO TO 900

      DO I = 1, NOUT
         RDATA(I) = QCAPP(I,2)
      ENDDO
      CALL MWTNUM('temperatureQCA',NCID,RECDIM,1,NRECS,MISS_P,RDATA,
     1            RCODE,MXMAP,
     2 RMAPDATA,IMAPDATA,I2MAPDATA,BMAPDATA)
      IF(RCODE.NE.SUCCESS_P)GO TO 900

      DO I = 1, NOUT
         RDATA(I) = QCRES(I,2)
      ENDDO
      CALL MWTNUM('temperatureQCR',NCID,RECDIM,1,NRECS,MISS_P,RDATA,
     1            RCODE,MXMAP,
     2 RMAPDATA,IMAPDATA,I2MAPDATA,BMAPDATA)
      IF(RCODE.NE.SUCCESS_P)GO TO 900

      DO I = 1, NOUT
         RDATA(I) = ICAPP(I,2)
      ENDDO
      CALL MWTNUM('temperatureICA',NCID,RECDIM,1,NRECS,MISS_P,RDATA,
     1            RCODE,MXMAP,
     2 RMAPDATA,IMAPDATA,I2MAPDATA,BMAPDATA)
      IF(RCODE.NE.SUCCESS_P)GO TO 900

      DO I = 1, NOUT
         RDATA(I) = ICRES(I,2)
      ENDDO
      CALL MWTNUM('temperatureICR',NCID,RECDIM,1,NRECS,MISS_P,RDATA,
     1            RCODE,MXMAP,
     2 RMAPDATA,IMAPDATA,I2MAPDATA,BMAPDATA)
      IF(RCODE.NE.SUCCESS_P)GO TO 900

      DO I = 1, NOUT
         RDATA(I) = RSAQC(I,2)
      ENDDO
      CALL MWTNUM('tempQcFlag',NCID,RECDIM,1,NRECS,MISS_P,RDATA,RCODE,
     1     MXMAP,
     2 RMAPDATA,IMAPDATA,I2MAPDATA,BMAPDATA)
      IF(RCODE.NE.SUCCESS_P)GO TO 900

C     ------------------------------------------------------------------
C     GET THE STATION NAMES & OB TIMES, AND CREATE/OUTPUT THE AWIPS
C     INVENTORY VARIABLES.
C     ------------------------------------------------------------------
      CALL MRDCHR('providerId',NCID,RECDIM,1,NRECS,INVID,I,RCODE)
      IF(RCODE.NE.0)GO TO 900

      CALL RDDBL('observationTime',NCID,RECDIM,1,NRECS,DMISS,INVTIME,
     1            RCODE)
      IF(RCODE.NE.0)GO TO 900

      CALL CDFINV(ATIME,NCID,NRECS,INVID,INVTIME,STATUS,
     1     MXTMPCI,MAX_IDLENCI,MAX_BINSCI,MAX_RECCI,LAST_RECCI,
     2     PREV_RECCI,INVENTORYCI,OFLOWCI,TIMBINCI,INDXCI,INVIDCI,
     3     BINVIDCI,STATIC_IDSCI,BSTATICCI,ITEMPCI,FIRST_IN_BINCI,
     4     LAST_IN_BINCI,LASTCI,FSL_P)

C     ------------------------------------------------------------------
C     CLOSE NETCDF FILE.
C     ------------------------------------------------------------------
 900  CALL NCCLOS(NCID,RCODE)

      RETURN
      END
