C$$$  SUBPROGRAM DOCUMENTATION BLOCK 
C                .      .    .                                       . 
C SUBPROGRAM:    CPSLABS     CP HYPERSLABS BETWEEN 2 DIFF NETCDF FILES
C   PRGMMR: M. BARTH         ORG: FSL        DATE: 01-02-06
C 
C ABSTRACT: FOR ALL VARIABLES, READ HYPERSLABS FROM INPUT NETCDF FILE
C   AND WRITE THEM TO THE NAMES HYPERSLABD IN THE OUTPUT NETCDF FILE
C 
C PROGRAM HISTORY LOG: 
C   01-02-06  M. BARTH    ORIGINAL VERSION
C   02-02-14  M. BARTH    ADDED OPTION TO ONLY FILTER "PAVEMENT" 
C                         VARIABLES.
C   05-03-14  L. BENJAMIN IMPLEMENTED THE USE OF RECFLTRWT FOR RECORD 
C                         TRANSFERS BETWEEN NETCDF FILES INSTEAD OF 
C                         VARIABLE TRANSFERS.
C   05-03-21  M. BARTH    ADDED DOPVFLTR ARGUMENT.
C 
C USAGE:    CALL RWSLABS (NCIDI,NCIDF,NVARS,RECDIM,NSLABS,STARTR,NRECS,
C                         ISFLTRPV,DOPVFLTR,PVFLTR,STATUS)
C
C   INPUT ARGUMENT LIST: 
C     NCIDI    - I       ID OF THE OPEN NETCDF INPUT FILE
C     NCIDF    - I       ID OF THE OPEN NETCDF OUTPUT FILE
C     NVARS    - I       NUMBER OF VARIABLES IN THE FILES
C     RECDIM   - I       ID OF THE RECORD DIMENSION
C     NSLABS   - I       NUMBER OF HYPERSLABS TO READ/WRITE
C     STARTR   - I       START RECORD FOR EACH HYPERSLAB
C     NRECS    - I       NUMBER OF RECORDS FOR EACH HYPERSLAB
C     DOPVFLTR - L       TRUE - FILTER PAVEMENT VARIABLES PER PVFLTR
C     PVFLTR   - L       TRUE - FILTER ONLY PAVEMENT VARIABLES FOR EACH
C                        RECORD
C 
C   OUTPUT ARGUMENT LIST:   
C     STATUS   - I       SUCCESS_P OR FAILURE_P 
C 
C   INPUT FILE:   
C     NETCDF FILE OPEN ON NCIDI
C
C   OUTPUT FILE:
C     NETCDF FILE OPEN ON NCIDF
C      
C SUBPROGRAMS CALLED: 
C     LIBRARY: - NETCDF LIBRARY:
C                  NCCLOS
C                  NCDINQ
C                  NCVGT
C                  NCVGTC
C                  NCVID
C                  NCVINQ
C                  NCVPT
C                  NCVPTC
C
C REMARKS:
C   OLD WAY --> IF VARIABLES ARE ADDED TO THE NETCDF FILE, OR 
C   VARIABLE NAMES ARE CHANGED WITHOUT WARNING, THIS MODULE WILL
C   OUTPUT A WARNING AND SKIP THE VARIABLE.  DYNAMIC CHANGES IN 
C   ATTRIBUTES, UNITS OR DIMENSIONS ARE NOT DETECTED.  IN OTHER 
C   WORDS, FOR THINGS TO WORK COMPLETELY AS PLANNED, THE CDL 
C   *MUST* BE A SUPERSET OF THE ACTUAL INPUT FILES.
C
C   NEW WAY --> THE INPUT AND OUTPUT FILES RECORD VARIABLES MUST
C   BE IDENTICAL (IN ORDER AND IN SIZE).  IF THEY ARE DIFFERNET, 
C   AN ERROR MESSAGE WILL BE RETURNED. 
C       
C   THE HYPERSLABS AREN'T CONTIGUOUS IN THE INPUT FILE, BUT WE FORCE
C   THEM TO BE CONTIGUOUS IN THE OUTPUT FILE:  THE WHOLE POINT IS
C   TO FILTER OUT STUFF FROM THE INPUT FILE.
C
C   ALSO NOTE THAT THE CODE CURRENTLY RESTRICTS PAVEMENT VARIABLES
C   TO HAVE VARIABLE NAMES THAT START WITH 'road' AND TO BE OF
C   NUMERIC DATA TYPES.
C
C   FOR ROAD VARIABLES FILTER CASES ONLY.  ROAD VARIABLES ARE 
C   RESTRICTED TO NCCHAR, NCFLOAT, AND NCSHORT.  ONLY 4 CONTINGUOUS  
C   ARRAYS OF NCFLOAT AND NCSHORT ARE ALLOWED.  REQUIRED ROAD 
C   VARIABLES FOR FILL VALUE INFORMATION ARE 'roadTemperature1' 
C   AND 'roadState1'.  ALL VARIABLES STARTING WITH 'road' ARE
C   ASSUMED TO BE ROAD VARIABLES.

C
C ATTRIBUTES: 
C   LANGUAGE: FORTRAN-77 + EXTENSIONS
C   MACHINE:  UNIX
C$$$ 


      SUBROUTINE RWSLABS (NCIDI,NCIDF,NVARS,RECDIM,NSLABS,STARTR,NRECS,
     1                    DOPVFLTR,PVFLTR,STATUS)


C     -----------------------------------------------------------------
C     INCLUDES AND COMMONS
C     -----------------------------------------------------------------
      INCLUDE 'IMPLICIT'
      INCLUDE 'MAPSCON'
      INCLUDE 'NETCDFI'
      INCLUDE 'STDOUT'
      INCLUDE 'INTQC1CON1'


C     -----------------------------------------------------------------
C     ARGUMENTS
C     -----------------------------------------------------------------
      INTEGER NCIDI,NCIDF,NVARS,RECDIM,NSLABS,STARTR(*),NRECS(*),STATUS
      LOGICAL DOPVFLTR,PVFLTR(*)

C     -----------------------------------------------------------------
C     LOCAL
C     - NOTE THE DIMENSION OF IDATA.  THE 6 IS TWICE THE CURRENT (8/98)
C       LARGEST SIZE OF MULTIPLE-DIMENSIONED NUMERIC VARIABLES.
C     - NOTE THE CHARACTER*512 DIMENSION OF CDATA:  512 IS THE SIZE OF
C       THE LARGEST NETCDF CHARACTER VARIABLE IN ANY OF THE FILES
C       HANDLED BY THIS ROUTINE.
C     -----------------------------------------------------------------
      INTEGER       I,J,K,N
      CHARACTER*(MAXNCNAM) VARNAM,DIMNAM
      INTEGER       NDIMS,NGATTS,DIMSIZ
      INTEGER       NVDIMS,VARTYP,LENSTR,NVATTS,RCODE,SAVE_LENSTR
      INTEGER       START(MAXVDIMS),COUNT(MAXVDIMS),VDIMS(MAXVDIMS)
      INTEGER       STARTRO
      DOUBLE PRECISION IDATA(MXST_P*6)
      CHARACTER*512 CDATA(MXST_P)

      CHARACTER*(MAXNCNAM) DIMNAMF
      INTEGER NRECI,OUTIDX,STATI,STATO,RECVARS,L
      INTEGER RECDIMF,NRECF,NDIMSF,NVARSF,NGATTSF

C IF THIS NUMBER CHANGES, THE "C" CALL HAS TO ALSO CHANGE
      INTEGER NUMRDARRAY
      PARAMETER (NUMRDARRAY = 4)
      
      INTEGER RDSTARTR(NUMRDARRAY),RDENDR(NUMRDARRAY)
      INTEGER RDSTARTI2(NUMRDARRAY),RDENDI2(NUMRDARRAY)
      INTEGER IVAR,CF,CS
      INTEGER*2 I2FILL
      REAL RFILL







C     *****************************************************************
C     ******************* BEGIN OF RWSLABS ****************************
C     *****************************************************************

C     -----------------------------------------------------------------
C     ASSUME FAILURE.
C     -----------------------------------------------------------------
      STATUS = FAILURE_P

C     -----------------------------------------------------------------
C     GET OUTPUT FILE RECORD DIMENSION AND GENERAL INFORMATION.
C     ALL EXISTING RECORDS ARE KEPT, THE FIRST NEW RECORD WILL
C     FOLLOW THE EXISTING RECORDS.
C     -----------------------------------------------------------------
      CALL NCINQ(NCIDF,NDIMSF,NVARSF,NGATTSF,RECDIMF,RCODE)
      IF(RCODE.NE.0)THEN
         WRITE(STDOUT,*)'  - NCINQ ERROR: ',RCODE
         GO TO 900
      ENDIF

      CALL NCDINQ(NCIDF,RECDIMF,DIMNAMF,NRECF,RCODE)
      IF(RCODE.NE.0)THEN
         WRITE(STDOUT,*)'  - NCDINQ RECDIM ERROR: ',RCODE
         GO TO 900
      ENDIF




C     -----------------------------------------------------------------
C     NO FILTERED ROAD VARIABLES CASE
C     -----------------------------------------------------------------
      IF(.NOT. DOPVFLTR)THEN

C     -----------------------------------------------------------------
C     INITIALIZE C ROUTINE FOR NETCDF RECORD COPY. 
C     -----------------------------------------------------------------
         CALL  RECFLTRWTNOFVINIT(NCIDI,RECVARS)

C SET NOFILL ON FOR SPEED
         L = NCSFIL(NCIDF, NCNOFILL, RCODE)

         OUTIDX = NRECF
         IF(OUTIDX.LT.0)OUTIDX = 0
         DO 611 J = 1, NSLABS
C     CHANGE INDEX TO C STANDARD FROM FORTRAN
            WRITE(*,*)'RECFLTRWTNOFV ',(STARTR(J)-1),
     1           OUTIDX,NRECS(J)
            CALL RECFLTRWTNOFV(NCIDI,NCIDF,%VAL(STARTR(J)-1),
     1           %VAL(OUTIDX),%VAL(NRECS(J)),STATI,STATO)
            IF(STATI.NE.SUCCESS_P)THEN
               WRITE(STDOUT,*)'ERROR READING INPUT FILE, IDX,RCODE: ',
     1              I,STATI
C SET FILL BACK ON
               L = NCSFIL(NCIDF, NCFILL, RCODE)
               GOTO 900
            ENDIF
            IF(STATO.NE.SUCCESS_P)THEN
               WRITE(STDOUT,*)'ERROR WRITING TO OUTPUT FILE, IDX,',
     1              'RCODE: ',(OUTIDX+1),STATO
C SET FILL BACK ON
               L = NCSFIL(NCIDF, NCFILL, RCODE)
               GOTO 900
            ENDIF
            OUTIDX = OUTIDX + NRECS(J)
 611     CONTINUE
C SET FILL BACK ON
         L = NCSFIL(NCIDF, NCFILL, RCODE)
         
         STATUS = SUCCESS_P
         GOTO 900
      ENDIF


C     -----------------------------------------------------------------
C     FILTERED ROAD VARIABLES case
C     -----------------------------------------------------------------


      CALL NCINQ(NCIDI,NDIMS,NVARS,NGATTS,RECDIM,RCODE)
      IF(RCODE.NE.0)THEN
         WRITE(STDOUT,*)'  - NCINQ ERROR: ',RCODE
         GO TO 900
      ENDIF

      CALL NCDINQ(NCIDI,RECDIM,DIMNAM,NRECI,RCODE)
      IF(RCODE.NE.0)THEN
         WRITE(STDOUT,*)'  - NCDINQ RECDIM ERROR: ',RCODE
         GO TO 900
      ENDIF



C     -----------------------------------------------------------------
C     GET ROAD VARIABLES INDEX DIMENSION AND FILL VALUES.  THE CODE IS
C     SET UP TO FILTER UP TO 4 CONTINGUOUS ARRAYS OF FLOAT'S AND 
C     SHORT'S ROAD VARIABLES. THIS CODE ONLY HANDLES ROAD VARIABLES
C     OF TYPE NCCHAR, NCFLOAT, OR NCSHORT.  THIS CODE ONLY HANDLES UP 
C     TO 4 CONTIGUOUS ARRAYS OF FLOATS AND SHORTS ROAD VARIABLES.
C     ALL VARIABLES STARTING WITH 'road' ARE ASSUMED TO BE ROAD 
C     VARIABLES.
C     -----------------------------------------------------------------

C     -----------------------------------------------------------------
C     INITIAL ROAD VARIABLE CONTIGUOUS ARRAYS TO MAX NETCDF VARIABLE 
C     VALUE.
C     -----------------------------------------------------------------
      DO I=1,4
C        REAL (FLOAT) ROAD VARIABLES
         RDSTARTR(I) = nf_max_vars
         RDENDR(I) = nf_max_vars
C        INTERGER*2 (SHORT) ROAD VARIABLES
         RDSTARTI2(I) = nf_max_vars
         RDENDI2(I) = nf_max_vars
      ENDDO

C     -----------------------------------------------------------------
C     CREATE ROAD VARIABLES ARRAYS INDEX (EACH CONTIBUOUS ARRAY START 
C     AND END RANGES).
C     -----------------------------------------------------------------
      CF = 1
      CS = 1
      DO I = 1, NVARS
         VARNAM = ' '
         CALL NCVINQ(NCIDI,I,VARNAM,VARTYP,NVDIMS,VDIMS,NVATTS,RCODE)
         IF(RCODE.NE.SUCCESS_P)THEN
            WRITE(STDOUT,*)'            - VARIABLE ',I,
     1                     ' NCVINQ ERROR: ',RCODE
            GO TO 900
         ENDIF

         K = NCVID(NCIDF,VARNAM,RCODE)
         

         IF(VARNAM(1:4).EQ.'road' .AND. VARTYP.NE.NCCHAR)THEN
            IF(VARTYP.EQ.NCFLOAT)THEN
               IF(RDSTARTR(CF).EQ.nf_max_vars)THEN
                  RDSTARTR(CF) = I
                  RDENDR(CF) = I
               ELSE IF(RDENDR(CF).EQ.I-1)THEN
                  RDENDR(CF) = I
               ELSE
                  CF = CF + 1
                  IF(CF.GT.NUMRDARRAY)THEN
                     WRITE(STDOUT,*)'THERE ARE MORE ROAD VARIABLE',
     1                    ' FLOAT SECTIONS THAN ALLOWED: ',CF,
     2                    NUMRDARRAY
                     GO TO 900
                  ENDIF
                  RDSTARTR(CF) = I
                  RDENDR(CF) = I
               ENDIF
            ELSE IF(VARTYP.EQ.NCSHORT)THEN
               IF(RDSTARTI2(CS).EQ.nf_max_vars)THEN
                  RDSTARTI2(CS) = I
                  RDENDI2(CS) = I
               ELSE IF(RDENDI2(CS).EQ.I-1)THEN
                  RDENDI2(CS) = I
               ELSE
                  CS = CS + 1
                  IF(CS.GT.NUMRDARRAY)THEN
                     WRITE(STDOUT,*)'THERE ARE MORE ROAD VARIABLE ',
     1                    'SHORT SECTIONS THAN ALLOWED: ',CS,NUMRDARRAY
                     GO TO 900
                  ENDIF
                  RDSTARTI2(CS) = I
                  RDENDI2(CS) = I
               ENDIF
            ELSE
               WRITE(STDOUT,*)'UNKNOWN ROAD VARIABLE FILL TYPE',
     1              ' ONLY FLOAT AND SHORT KNOWN: ',VARTYP
               GO TO 900
            ENDIF
         ENDIF
      ENDDO


C     -----------------------------------------------------------------
C     GET ROAD VARIABLES FLOAT AND SHORT FILL VALUES
C     -----------------------------------------------------------------
      IVAR = NCVID(NCIDF,'roadTemperature1',RCODE)
      IF(RCODE.NE.SUCCESS_P)THEN
         WRITE(STDOUT,*)'ERROR READING roadTemperature1 INDEX #. ',
     1        RCODE
         RETURN
      ENDIF

      CALL NCAGT(NCIDF,IVAR,'_FillValue',RFILL,RCODE)
      IF(RCODE.NE.SUCCESS_P)THEN
         WRITE(STDOUT,*)'ERROR READING roadTemperature1 FILL VALUE. ',
     1        RCODE
         RETURN
      ENDIF

      IVAR = NCVID(NCIDF,'roadState1',RCODE)
      IF(RCODE.NE.SUCCESS_P)THEN
         WRITE(STDOUT,*)'ERROR READING roadState1 INDEX #. ',
     1        RCODE
         RETURN
      ENDIF

      CALL NCAGT(NCIDF,IVAR,'_FillValue',I2FILL,RCODE)
      IF(RCODE.NE.SUCCESS_P)THEN
         WRITE(STDOUT,*)'ERROR READING roadState1 FILL VALUE. ',
     1        RCODE
         RETURN
      ENDIF


C     -----------------------------------------------------------------
C     INITIALIZE C ROUTINE FOR NETCDF RECORD COPY.  THE C INDEX
C     STARTS AT 0 INSTEAD OF 1, SO ADJUST OUR INDEXES TO FIT C
C     STANDARDS.
C     -----------------------------------------------------------------
      CALL  RECFLTRWTINIT(NCIDI,
     1     %VAL(RDSTARTR(1)-1),%VAL(RDENDR(1)-1),
     2     %VAL(RDSTARTI2(1)-1),%VAL(RDENDI2(1)-1),
     3     %VAL(RDSTARTR(2)-1),%VAL(RDENDR(2)-1),
     4     %VAL(RDSTARTI2(2)-1),%VAL(RDENDI2(2)-1),
     5     %VAL(RDSTARTR(3)-1),%VAL(RDENDR(3)-1),
     6     %VAL(RDSTARTI2(3)-1),%VAL(RDENDI2(3)-1),
     5     %VAL(RDSTARTR(4)-1),%VAL(RDENDR(4)-1),
     6     %VAL(RDSTARTI2(4)-1),%VAL(RDENDI2(4)-1),
     7     %VAL(RFILL),%VAL(I2FILL),RECVARS)



C     -----------------------------------------------------------------
C     "C" IS INDEX TO 0, SO START OUTIDX AT "C" INDEXES STARTING POINT
C     -----------------------------------------------------------------
      OUTIDX = NRECF
      IF(OUTIDX.LT.0)OUTIDX = 0
      DO 61 J = 1, NSLABS
         DO 53 I = STARTR(J), STARTR(J)+NRECS(J)-1
         CALL RECFLTRWT(NCIDI,NCIDF,%VAL(I-1),%VAL(OUTIDX),
     1        %VAL(PVFLTR(I)),STATI,STATO)

            IF(STATI.NE.SUCCESS_P)THEN
               WRITE(STDOUT,*)'ERROR READING INPUT FILE, IDX,RCODE: ',
     1                        I,STATI
               GOTO 900
            ENDIF
            IF(STATO.NE.SUCCESS_P)THEN
               WRITE(STDOUT,*)'ERROR WRITING TO OUTPUT FILE, IDX,',
     1                        'RCODE: ',(OUTIDX+1),STATO
               GOTO 900
            ENDIF

C     -----------------------------------------------------------------
C     INCREMENT THE "C" OUTPUT INDEX
C     -----------------------------------------------------------------

            OUTIDX = OUTIDX + 1

 53      CONTINUE
 61   CONTINUE
      STATUS = SUCCESS_P

 900  CALL NCCLOS(NCIDI,RCODE)

      RETURN
      END
