C$$$  SUBPROGRAM DOCUMENTATION BLOCK
C                .      .    .                                       .
C SUBPROGRAM:    M_RDISLL    CALCULATE DISTANCE IN MILES FROM LAT/LON
C   PRGMMR: M. BARTH         ORG: FSL        DATE: 00-06-20
C
C ABSTRACT: CALCULATE THE DISTANCE IN MILES ON A SPHERICAL EARTH
C   GIVEN LATITUDE AND LONGITUDE COORDINATES.
C
C PROGRAM HISTORY LOG:
C   00-06-20  M. BARTH    ORIGINAL VERSION
C   08-08-22  L. BENJAMIN ADDED COMMENT (NAUTICAL MILES)
C
C USAGE:    DIS = M_RDISLL(LAT1,LON1,LAT2,LON2,SPVAL)
C
C   INPUT ARGUMENT LIST:
C     LAT1     - R       LATITUDE (DEG N)
C     LON1     - R       LONGITUDE (DEG E)
C     LAT2     - R       LATITUDE (DEG N)
C     LON2     - R       LONGITUDE (DEG E)
C     SPVAL    - R       MISSING DATA FLAG

C   OUTPUT ARGUMENT LIST:
C     M_RDISLL - R       DISTANCE IN MILES BETWEEN THE TWO POINTS
C                        OR SPVAL IF ONE OF THE ARGS IS SPVAL
C
C SUBPROGRAMS CALLED:
C     NONE
C
C REMARKS:
C
C   THE DISTANCE CALCULATION USED IN THIS ROUTINE IS DONE USING
C   THE HAVERSINE FORMULA EXPRESSED IN TERMS OF A TWO-ARGUMENT
C   INVERSE TANGENT FUNCTION.  THE HAVERSINE FORMULA REFERENCE IS:
C   R.W. SINNOTT, "VIRTUES OF THE HAVERSINE", SKY AND TELESCOPE,
C   VOL. 68, NO. 2, 1984, P. 159.  A WEB PAGE DISCUSSING THE
C   VARIOUS ALGORITHMS USED TO CALCULATE DISTANCE ON THE EARTH
C   (AND WHERE I FOUND THE HAVERSINE FORMULA) IS AT
C   www.census.gov/cgi-bin/geo/gisfaq?Q5.1
C
C   WHEN THE TWO POINTS ARE ANTIPODAL (ON OPPOSITE SIDES OF THE
C   EARTH) THE HAVERSINE FORMULA IS ILL-CONDITIONED, BUT THE ERROR,
C   PERHAPS AS LARGE AS 1 MILE, IS IN THE CONTEXT OF A DISTANCE
C   NEAR 12,000 MILES.
C
C ATTRIBUTES:
C   LANGUAGE: FORTRAN-77 + EXTENSIONS
C   MACHINE:  UNIX
C$$$

      REAL FUNCTION M_RDISLL(LAT1,LON1,LAT2,LON2,SPVAL)

C     ------------------------------------------------------------------
C     INCLUDES AND COMMONS
C     ------------------------------------------------------------------
      INCLUDE 'IMPLICIT'

C     ------------------------------------------------------------------
C     ARGUMENTS
C     ------------------------------------------------------------------
      REAL LAT1,LON1,LAT2,LON2,SPVAL

C     ------------------------------------------------------------------
C     LOCAL
C     ------------------------------------------------------------------
      REAL A,C,R,RADDEG,DLAT,DLON,LA1,LA2,LO1,LO2

C     R IS RADIUS OF THE EARTH IN NAUTICAL MILES
C     RADDEG CONVERTS DEGREES TO RADIANS

      PARAMETER (R = 3956.0)
      PARAMETER (RADDEG = 1.745329E-2)

C     *****************************************************************
C     ******************* BEGIN OF M_RDISLL ***************************
C     *****************************************************************
      IF(LAT1.EQ.SPVAL.OR.LAT2.EQ.SPVAL.OR.LON1.EQ.SPVAL.OR.
     1   LON2.EQ.SPVAL)THEN

         M_RDISLL = SPVAL

      ELSE

         LA1 = LAT1 * RADDEG
         LA2 = LAT2 * RADDEG
         LO1 = LON1 * RADDEG
         LO2 = LON2 * RADDEG

         DLAT = LA2 - LA1
         DLON = LO2 - LO1

         A = SIN(DLAT/2)**2 + COS(LA1) * COS(LA2) * SIN(DLON/2)**2
         C = 2.0 * ATAN2(SQRT(A),SQRT(1.0-A))
         M_RDISLL = R * C

      ENDIF

      RETURN
      END
C**---------------------------------------------------------------------
C**REQUIRED STANDARD FSL DISCLAIMER (NOVEMBER 2000)
C**---------------------------------------------------------------------
C**  OPEN SOURCE LICENSE/DISCLAIMER,  FORECAST SYSTEMS LABORATORY
C**  NOAA/OAR/FSL, 325 BROADWAY BOULDER, CO 80305
C**
C**  THIS SOFTWARE IS DISTRIBUTED UNDER THE OPEN SOURCE DEFINITION, 
C**  WHICH MAY BE FOUND AT http://www.opensource.org/osd.html.
C**
C**  IN PARTICULAR, REDISTRIBUTION AND USE IN SOURCE AND BINARY FORMS,
C**  WITH OR WITHOUT MODIFICATION, ARE PERMITTED PROVIDED THAT THE 
C**  FOLLOWING CONDITIONS ARE MET:
C**
C**  - REDISTRIBUTIONS OF SOURCE CODE MUST RETAIN THIS NOTICE, THIS LIST
C**    OF CONDITIONS AND THE FOLLOWING DISCLAIMER.
C**
C**  - REDISTRIBUTIONS IN BINARY FORM MUST PROVIDE ACCESS TO THIS 
C**    NOTICE, THIS LIST OF CONDITIONS AND THE FOLLOWING DISCLAIMER, AND
C**    THE UNDERLYING SOURCE CODE.
C**
C**  - ALL MODIFICATIONS TO THIS SOFTWARE MUST BE CLEARLY DOCUMENTED, 
C**    AND ARE SOLELY THE RESPONSIBILITY OF THE AGENT MAKING THE 
C**    MODIFICATIONS.
C**
C**  - IF SIGNIFICANT MODIFICATIONS OR ENHANCEMENTS ARE MADE TO THIS
C**    SOFTWARE, THE FSL SOFTWARE POLICY MANAGER 
C**    (softwaremgr.fsl@noaa.gov) BE NOTIFIED.
C**
C**  THIS SOFTWARE AND ITS DOCUMENTATION ARE IN THE PUBLIC DOMAIN AND 
C**  ARE FURNISHED "AS IS."  THE AUTHORS, THE UNITED STATES GOVERNMENT,
C**  ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO 
C**  WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE
C**  AND DOCUMENTATION FOR ANY PURPOSE.  THEY ASSUME NO RESPONSIBILITY 
C**  (1) FOR THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO 
C**  PROVIDE TECHNICAL SUPPORT TO USERS.
C**---------------------------------------------------------------------
