C$$$  SUBPROGRAM DOCUMENTATION BLOCK
C                .      .    .                                       .
C SUBPROGRAM:    M_RDISLLM   CALCULATE DISTANCE IN METERS FROM LAT/LON
C   PRGMMR: M. BARTH         ORG: FSL        DATE: 08-08-22
C
C ABSTRACT: CALCULATE THE DISTANCE IN METERS ON A SPHERICAL EARTH
C   GIVEN LATITUDE AND LONGITUDE COORDINATES.
C
C PROGRAM HISTORY LOG:
C   08-08-22  L. BENJAMIN ORIGINAL VERSION CLONED FROM M_RDISLL
C   09-08-07  (07 Aug 2009) T. Schlatter corrected distance formula, 
C   revised remarks on documentation, and changed Earth radius to 
C   standard value.
C
C USAGE:    DIS = M_RDISLLM(LAT1,LON1,LAT2,LON2,SPVAL)
C
C   LATITUDES AND LONGITUDES ARE IN DECIMAL DEGREES.
C   INPUT ARGUMENT LIST:
C     LAT1     - R       LATITUDE (DEG N) (-90 to +90 degrees)
C     LON1     - R       LONGITUDE (DEG E) (-180 to +180 degrees)
C     LAT2     - R       LATITUDE (DEG N)
C     LON2     - R       LONGITUDE (DEG E)
C     SPVAL    - R       MISSING DATA FLAG
C
C   OUTPUT ARGUMENT LIST:
C     M_RDISLLM - R      DISTANCE IN METERS BETWEEN THE TWO POINTS
C                        OR SPVAL IF ONE OF THE ARGS IS SPVAL
C
C SUBPROGRAMS CALLED:
C     NONE
C
C REMARKS:
C
C   THE DISTANCE CALCULATION USED IN THIS ROUTINE MAKES USE OF
C   SPHERICAL TRIGONOMETRY.  THE PARTICULAR FORMULA USED
C   APPEARS AT http://en.wikipedia.org/wiki/Great_circle
C   (THE MIDDLE FORMULA OF THREE).
C   A STANDARD QUICK REFERENCE ON SPHERICAL TRIGONOMETRY IS 
C   "MATHEMATICAL HANDBOOK FOR SCIENTISTS AND ENGINEERS" BY
C   G.A. KORN AND T.M. KORN, MCGRAW-HILL, NEW YORK, 1968,
C   PP. 888-893.  THIS FORMULA IS EXACT.  IT IS CHOSEN 
C   BECAUSE COMPUTER ROUNDOFF ERROR IS VERY SMALL WHEN
C   THE TWO POINTS ON A SPHERE ARE CLOSE TOGETHER.  THE
C   FORMULA SUFFERS FROM ROUNDOFF ERROR ONLY WHEN THE TWO
C   POINTS ARE NEARLY OPPOSITE EACH OTHER ON THE SPHERE.
C  
C
C ATTRIBUTES:
C   LANGUAGE: FORTRAN-77 + EXTENSIONS
C   MACHINE:  UNIX
C$$$

      REAL FUNCTION M_RDISLLM(LAT1,LON1,LAT2,LON2,SPVAL)

C     ------------------------------------------------------------------
C     INCLUDES AND COMMONS
C     ------------------------------------------------------------------
      INCLUDE 'IMPLICIT'

C     ------------------------------------------------------------------
C     ARGUMENTS
C     ------------------------------------------------------------------
      REAL LAT1,LON1,LAT2,LON2,SPVAL

C     ------------------------------------------------------------------
C     LOCAL
C     ------------------------------------------------------------------
      REAL A,C,R,RADDEG,DLAT,DLON,LA1,LA2,LO1,LO2

C     R IS RADIUS OF THE EARTH IN METERS
C     RADDEG CONVERTS DEGREES TO RADIANS

      PARAMETER (R = 6.371E06)
      PARAMETER (RADDEG = 1.745329E-2)

C     *****************************************************************
C     ******************* BEGIN OF M_RDISLLM **************************
C     *****************************************************************
      IF(LAT1.EQ.SPVAL.OR.LAT2.EQ.SPVAL.OR.LON1.EQ.SPVAL.OR.
     1   LON2.EQ.SPVAL)THEN

         M_RDISLLM = SPVAL

      ELSE

         LA1 = LAT1 * RADDEG
         LA2 = LAT2 * RADDEG
         LO1 = LON1 * RADDEG
         LO2 = LON2 * RADDEG

         DLAT = LA2 - LA1
         DLON = LO2 - LO1

         A = SIN(DLAT/2)**2 + COS(LA1) * COS(LA2) * SIN(DLON/2)**2
         C = 2.0 * ASIN(SQRT(A))
         M_RDISLLM = R * C

      ENDIF

      RETURN
      END
C**---------------------------------------------------------------------
C**REQUIRED STANDARD FSL DISCLAIMER (NOVEMBER 2000)
C**---------------------------------------------------------------------
C**  OPEN SOURCE LICENSE/DISCLAIMER,  FORECAST SYSTEMS LABORATORY
C**  NOAA/OAR/FSL, 325 BROADWAY BOULDER, CO 80305
C**
C**  THIS SOFTWARE IS DISTRIBUTED UNDER THE OPEN SOURCE DEFINITION, 
C**  WHICH MAY BE FOUND AT http://www.opensource.org/osd.html.
C**
C**  IN PARTICULAR, REDISTRIBUTION AND USE IN SOURCE AND BINARY FORMS,
C**  WITH OR WITHOUT MODIFICATION, ARE PERMITTED PROVIDED THAT THE 
C**  FOLLOWING CONDITIONS ARE MET:
C**
C**  - REDISTRIBUTIONS OF SOURCE CODE MUST RETAIN THIS NOTICE, THIS LIST
C**    OF CONDITIONS AND THE FOLLOWING DISCLAIMER.
C**
C**  - REDISTRIBUTIONS IN BINARY FORM MUST PROVIDE ACCESS TO THIS 
C**    NOTICE, THIS LIST OF CONDITIONS AND THE FOLLOWING DISCLAIMER, AND
C**    THE UNDERLYING SOURCE CODE.
C**
C**  - ALL MODIFICATIONS TO THIS SOFTWARE MUST BE CLEARLY DOCUMENTED, 
C**    AND ARE SOLELY THE RESPONSIBILITY OF THE AGENT MAKING THE 
C**    MODIFICATIONS.
C**
C**  - IF SIGNIFICANT MODIFICATIONS OR ENHANCEMENTS ARE MADE TO THIS
C**    SOFTWARE, THE FSL SOFTWARE POLICY MANAGER 
C**    (softwaremgr.fsl@noaa.gov) BE NOTIFIED.
C**
C**  THIS SOFTWARE AND ITS DOCUMENTATION ARE IN THE PUBLIC DOMAIN AND 
C**  ARE FURNISHED "AS IS."  THE AUTHORS, THE UNITED STATES GOVERNMENT,
C**  ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO 
C**  WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE
C**  AND DOCUMENTATION FOR ANY PURPOSE.  THEY ASSUME NO RESPONSIBILITY 
C**  (1) FOR THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO 
C**  PROVIDE TECHNICAL SUPPORT TO USERS.
C**---------------------------------------------------------------------

