C$$$  SUBPROGRAM DOCUMENTATION BLOCK
C                .      .    .                                       .
C SUBPROGRAM:    UATEMPVC    APPLY UPPER AIR VALIDITY CHECK TO TEMPS
C   PRGMMR: M. BARTH         ORG: FSL        DATE: 00-03-14
C
C ABSTRACT: APPLY THE TSP-SPECIFIED UPPER AIR VALIDITY CHECK TO
C   TEMPERATURES.
C
C PROGRAM HISTORY LOG:
C   00-03-14  M. BARTH    ORIGINAL VERSION
C   02-04-25  M. BARTH    ALLOWED PRS OR HT COORDINATE
C   06-08-15  L. BENJAMIN CHANGED COORD AND T CHECK TO EQUAL MISS_P
C                         INSTEAD OF GREATER THAN AND EQUAL.
C   08-06-13  L. BENJAMIN CHANGED MADISCON INCLUDE TO INTQC1CON2
C
C USAGE:    IVAL = UATEMPVC(CTYPE,COORD,T)
C
C   INPUT ARGUMENT LIST:
C     CTYPE    - I       1 - COORD IS HEIGHT (M ABOVE MSL)
C                        2 - COORD IS PRESSURE (MB)
C     COORD    - R       HEIGHT/PRESSURE OF TEMP TO BE CHECKED
C     T        - R       TEMPERATURE (K) TO BE CHECKED
C
C   OUTPUT ARGUMENT LIST:
C     IVAL     - I       0 - PASSED THE VALIDITY CHECK
C                        1 - FAILED THE VALIDITY CHECK
C                       -1 - ONE OR MORE ARGS ARE MISSING
C
C SUBPROGRAMS CALLED:
C     MADISLIB:  M_ZTOPSA
C
C REMARKS:
C   THE VALIDITY CHECK FOR UPPER AIR DATA IN SPECIFIED IN AWIPS
C   TSP-032-1992R1, TABLE A4-1.  NOTE THAT THE THRESHOLDS ARE IN
C   DEGREES C BUT INPUTS ARE IN DEGREES K.
C
C ATTRIBUTES:
C   LANGUAGE: FORTRAN-77 + EXTENSIONS
C   MACHINE:  UNIX
C$$$

      INTEGER FUNCTION UATEMPVC(CTYPE,COORD,T)

C     ------------------------------------------------------------------
C     INCLUDES AND COMMONS
C     ------------------------------------------------------------------
      INCLUDE 'IMPLICIT'
      INCLUDE 'INTQC1CON2'

C     ------------------------------------------------------------------
C     ARGUMENTS
C     ------------------------------------------------------------------
      INTEGER CTYPE
      REAL COORD,T

C     ------------------------------------------------------------------
C     LOCAL
C     ------------------------------------------------------------------
      INTEGER PLEV(15),MINM(15),MAXM(15),I,LASTL,LASTH,NEXTL,NEXTH
      REAL M_ZTOPSA,PRS

C     ------------------------------------------------------------------
C     DATA STATEMENTS (KEEP AT BOTTOM OF DECLARATIONS)
C     ------------------------------------------------------------------
      DATA PLEV/1000,850,700,500,400,300,250,200,150,100,70,50,30,20,10/
      DATA MINM/-65,-50,-50,-57,-66,-72,-76,-78,-85,-95,-95,-95,-95,
     1          -95,-95/
      DATA MAXM/60,45,30,5,-10,-20,-25,-30,-30,-30,-25,-15,-5,5,15/

C     ------------------------------------------------------------------
C     IF ONE OR MORE OF THE ARGUMENTS IS MISSING SET RETURN CODE AND
C     EXIT.
C     ------------------------------------------------------------------
      IF(COORD.EQ.MISS_P.OR.T.EQ.MISS_P)THEN
         UATEMPVC = -1
         RETURN
      ENDIF

C     ------------------------------------------------------------------
C     IF THE USER IS SPECIFYING HEIGHT, CONVERT IT TO PRESSURE USING
C     A STANDARD ATMOSPHERE.
C     ------------------------------------------------------------------
      IF(CTYPE.EQ.1)THEN
         PRS = M_ZTOPSA(COORD,MISS_P)
      ELSE
         PRS = COORD
      ENDIF

C     ------------------------------------------------------------------
C     FIND THE RIGHT PAIR IN THE TABLE TO USE, THEN USE THE MAXIMUM
C     OF THE PAIR AND MAKE THE COMPARISON (IN DEG C).
C     ------------------------------------------------------------------
      LASTL = MINM(1)
      LASTH = MAXM(1)

      DO 1 I = 1, 15
         IF(PRS.GE.PLEV(I))THEN
            NEXTL = MINM(I)
            NEXTH = MAXM(I)
            IF(PRS.EQ.PLEV(I))THEN
               LASTL = NEXTL
               LASTH = NEXTH
            ENDIF
            GO TO 10
         ENDIF
         LASTL = MINM(I)
         LASTH = MAXM(I)
 1    CONTINUE

      NEXTL = MINM(15)
      NEXTH = MAXM(15)

 10   IF(T-273.15.LT.MIN(LASTL,NEXTL).OR.
     1   T-273.15.GT.MAX(LASTH,NEXTH))THEN
         UATEMPVC = 1
      ELSE
         UATEMPVC = 0
      ENDIF

      RETURN
      END
