C$$$  SUBPROGRAM DOCUMENTATION BLOCK
C                .      .    .                                       .
C SUBPROGRAM:    UAWINDVC    APPLY UPPER AIR VALIDITY CHECK TO WINDS
C   PRGMMR: M. BARTH         ORG: FSL        DATE: 00-02-01
C
C ABSTRACT: APPLY THE TSP-SPECIFIED UPPER AIR VALIDITY CHECK TO
C   WIND SPEED AND DIRECTION.
C
C PROGRAM HISTORY LOG:
C   00-02-01  M. BARTH    ORIGINAL VERSION
C   06-08-15  L. BENJAMIN CHANGED COORD AND SPD AND DIR CHECK TO EQUAL 
C                         MISS_P INSTEAD OF GREATER THAN AND EQUAL.
C   08-06-13  L. BENJAMIN CHANGED MADISCON INCLUDE TO INTQC1CON2
C
C USAGE:    IVAL = UAWINDVC(CTYPE,COORD,SPD,DIR)
C
C   INPUT ARGUMENT LIST:
C     CTYPE    - I       1 - COORD IS HEIGHT (M ABOVE MSL)
C                        2 - COORD IS PRESSURE (MB)
C     COORD    - R       HEIGHT/PRESSURE OF WIND TO BE CHECKED
C     SPD      - R       WIND SPEED (M/S)
C     DIR      - R       WIND DIRECTION (DEG FROM TRUE NORTH)
C
C   OUTPUT ARGUMENT LIST:
C     IVAL     - I       0 - PASSED THE VALIDITY CHECK
C                        1 - SPD FAILED THE VALIDITY CHECK
C                        2 - DIR FAILED THE VALIDITY CHECK
C                        3 - BOTH FAILED THE VALIDITY CHECK
C                       -1 - ONE OR MORE ARGS ARE MISSING
C
C SUBPROGRAMS CALLED:
C     UNIQUE:    M_ZTOPSA
C
C REMARKS:
C   THE VALIDITY CHECK FOR UPPER AIR DATA IN SPECIFIED IN AWIPS
C   TSP-032-1992R1, TABLE A4-1.
C
C ATTRIBUTES:
C   LANGUAGE: FORTRAN-77 + EXTENSIONS
C   MACHINE:  UNIX
C$$$

      INTEGER FUNCTION UAWINDVC(CTYPE,COORD,SPD,DIR)

C     ------------------------------------------------------------------
C     INCLUDES AND COMMONS
C     ------------------------------------------------------------------
      INCLUDE 'IMPLICIT'
      INCLUDE 'INTQC1CON2'

C     ------------------------------------------------------------------
C     ARGUMENTS
C     ------------------------------------------------------------------
      INTEGER CTYPE
      REAL COORD,SPD,DIR

C     ------------------------------------------------------------------
C     LOCAL
C     ------------------------------------------------------------------
      REAL M_ZTOPSA,KTS,PRS
      INTEGER PLEV(15),MAXKTS(15),I,LAST,NEXT
      LOGICAL BADSPD,BADDIR

C     ------------------------------------------------------------------
C     DATA STATEMENTS (KEEP AT BOTTOM OF DECLARATIONS)
C     ------------------------------------------------------------------
      DATA PLEV/1000,850,700,500,400,300,250,200,150,100,70,50,30,20,10/
      DATA MAXKTS/70,90,120,200,250,300,300,300,200,200,200,200,
     1            200,200,200/

C     ------------------------------------------------------------------
C     IF ONE OR MORE OF THE ARGUMENTS IS MISSING, SET RETURN CODE AND
C     EXIT.
C     ------------------------------------------------------------------
      IF(SPD.EQ.MISS_P.OR.DIR.EQ.MISS_P.OR.COORD.EQ.MISS_P)THEN
         UAWINDVC = -1
         RETURN
      ENDIF

C     ------------------------------------------------------------------
C     IF THE USER IS SPECIFYING HEIGHT, CONVERT IT TO PRESSURE USING
C     A STANDARD ATMOSPHERE.
C     ------------------------------------------------------------------
      IF(CTYPE.EQ.1)THEN
         PRS = M_ZTOPSA(COORD,MISS_P)
      ELSE
         PRS = COORD
      ENDIF

C     ------------------------------------------------------------------
C     FIND THE RIGHT PAIR IN THE TABLE TO USE, THEN USE THE MAXIMUM
C     OF THE PAIR AND MAKE THE COMPARISON (IN KTS).
C     ------------------------------------------------------------------
      LAST = MAXKTS(1)

      DO 1 I = 1, 15
         IF(PRS.GE.PLEV(I))THEN
            NEXT = MAXKTS(I)
            IF(PRS.EQ.PLEV(I))LAST = NEXT
            GO TO 10
         ENDIF
         LAST = MAXKTS(I)
 1    CONTINUE

      NEXT = MAXKTS(15)

 10   KTS = SPD * 1.943861286
      IF(KTS.GT.MAX(LAST,NEXT))THEN
         BADSPD = .TRUE.
      ELSE
         BADSPD = .FALSE.
      ENDIF

      IF(DIR.LT.0.OR.DIR.GT.360)THEN
         BADDIR = .TRUE.
      ELSE
         BADDIR = .FALSE.
      ENDIF

      IF(BADSPD.AND.BADDIR)THEN
         UAWINDVC = 3
      ELSE IF(BADSPD)THEN
         UAWINDVC = 1
      ELSE IF(BADDIR)THEN
         UAWINDVC = 2
      ELSE
         UAWINDVC = 0
      ENDIF

      RETURN
      END
